searchData={"items":[{"type":"module","title":"VintageNet","doc":"`VintageNet` is network configuration library built specifically for [Nerves\nProject](https://nerves-project.org) devices. It has the following features:\n\n* Ethernet and WiFi support included. Extendible to other technologies\n* Default configurations specified in your Application config\n* Runtime updates to configurations are persisted and applied on next boot (can\n  be disabled)\n* Simple subscription to network status change events\n* Connect to multiple networks at a time and prioritize which interfaces are\n  used (Ethernet over WiFi over cellular)\n* Internet connection monitoring and failure detection (currently slow and\n  simplistic)\n\nSee\n[github.com/nerves-networking/vintage_net](https://github.com/nerves-networking/vintage_net)\nfor more information.","ref":"VintageNet.html"},{"type":"function","title":"VintageNet.all_interfaces/0","doc":"Return a list of all interfaces on the system","ref":"VintageNet.html#all_interfaces/0"},{"type":"function","title":"VintageNet.configuration_valid?/2","doc":"Check if this is a valid configuration\n\nThis runs the validation routines for a settings map, but doesn't try to\napply them.","ref":"VintageNet.html#configuration_valid?/2"},{"type":"function","title":"VintageNet.configure/3","doc":"Update the configuration of a network interface\n\nConfigurations are validated and normalized before being applied.  This means\nthat type errors and missing required fields will be caught and old or\nredundant ways of specifying configurations will be fixed.  Call\n`get_configuration/1` to see how what changes, if any, were made as part of\nthe normalization process.\n\nAfter validation, the configuration is optionally persisted and applied.\n\nSee the `VintageNet` documentation for configuration examples or your\n`VintageNet.Technology` provider's docs.\n\nOptions:\n\n* `:persist` - set to `false` to avoid persisting this configuration. System\n  restarts will revert to the previous configuration. Defaults to true.","ref":"VintageNet.html#configure/3"},{"type":"function","title":"VintageNet.configured_interfaces/0","doc":"Return a list of configured interface","ref":"VintageNet.html#configured_interfaces/0"},{"type":"function","title":"VintageNet.deconfigure/2","doc":"Deconfigure and persists (by default) settings for a specified interface.\n\nSupports same options as `configure/3`","ref":"VintageNet.html#deconfigure/2"},{"type":"function","title":"VintageNet.get/2","doc":"Get the current value of a network property\n\nSee `get_by_prefix/1` for exact prefix matches (i.e., get all properties for one\ninterface) and `match/1` to run wildcard matches (i.e., get a specific\nproperty for all interfaces).","ref":"VintageNet.html#get/2"},{"type":"function","title":"VintageNet.get_by_prefix/1","doc":"Get a list of all properties matching the specified prefix\n\nTo get a list of all known properties and their values, call\n`VintageNet.get_by_prefix([])`","ref":"VintageNet.html#get_by_prefix/1"},{"type":"function","title":"VintageNet.get_configuration/1","doc":"Return the settings for the specified interface","ref":"VintageNet.html#get_configuration/1"},{"type":"function","title":"VintageNet.info/1","doc":"Print the current network status\n\nOptions include:\n\n* `:redact` - Set to `false` to print out passwords","ref":"VintageNet.html#info/1"},{"type":"function","title":"VintageNet.ioctl/3","doc":"Run a command on a network interface\n\nCommands are mostly network interface-specific. Also see the `VintageNet`\nPropertyTable fo getting status or registering for status changes.","ref":"VintageNet.html#ioctl/3"},{"type":"function","title":"VintageNet.match/1","doc":"Get a list of all properties matching a pattern\n\nPatterns are list of strings that optionally specify `:_` at\na position in the list to match any value.","ref":"VintageNet.html#match/1"},{"type":"function","title":"VintageNet.max_interface_count/0","doc":"Return the maximum number of interfaces controlled by VintageNet\n\nInternal constraints mean that VintageNet can't manage an arbitrary number of\ninterfaces and knowing the max can reduce some processing. The limit is set\nby the application config. Unless you need over 100 network interfaces,\nVintageNet's use of the Linux networking API is not likely to be an issue,\nthough.","ref":"VintageNet.html#max_interface_count/0"},{"type":"function","title":"VintageNet.reset_to_defaults/1","doc":"Configure an interface to use the defaults\n\nThis configures an interface to the defaults found in the application\nenvironment (`config.exs`). If the application environment doesn't have a\ndefault configuration, the interface is deconfigured. On reboot, the\ninterface will continue to use the defaults and if a new version of firmware\nupdates the defaults, it will use those.","ref":"VintageNet.html#reset_to_defaults/1"},{"type":"function","title":"VintageNet.scan/1","doc":"Initiate an access point scan on a wireless interface\n\nThe scan results are posted asynchronously to the `[\"interface\", ifname, \"wifi\", \"access_points\"]`\nproperty as they come in. After waiting a second or two they can be fetched via\n`VintageNet.get([\"interface\", ifname, \"wifi\", \"access_points\"])`.\n\nIt appears that there's some variation in how scanning is implemented on WiFi adapters. One\nstrategy that seems to work is to call `scan/1` every 10 seconds or so while prompting a user to\npick a WiFi network.\n\nThis is a utility function for calling the `:scan` ioctl.","ref":"VintageNet.html#scan/1"},{"type":"function","title":"VintageNet.subscribe/1","doc":"Subscribe to property change messages\n\nMessages have the form:\n\n```\n{VintageNet, property_name, old_value, new_value, metadata}\n```\n\nSubscriptions are prefix matches. For example, to get notified whenever a property\nchanges on \"wlan0\", run this:\n\n```\nVintageNet.subscribe([\"interface\", \"wlan0\"])\n```\n\nIt's also possible to match with wildcards using `:_`. For example, to\nget notified whenever an IP address in the system changes, do this:\n\n```\nVintageNet.subscribe([\"interface\", :_, \"addresses\"])\n```","ref":"VintageNet.html#subscribe/1"},{"type":"function","title":"VintageNet.unsubscribe/1","doc":"Stop subscribing to property change messages","ref":"VintageNet.html#unsubscribe/1"},{"type":"function","title":"VintageNet.verify_system/1","doc":"Check that the system has the required programs installed\n\nNOTE: This isn't completely implemented yet!","ref":"VintageNet.html#verify_system/1"},{"type":"type","title":"VintageNet.any_ip_address/0","doc":"IP addresses in VintageNet can be specified as strings or tuples\n\nWhile VintageNet uses IP addresses in tuple form internally, it can be\ncumbersome to always convert to tuple form in practice. The general rule is\nthat VintageNet is flexible in how it accepts IP addresses, but if you get an\naddress from a VintageNet API, it will be in tuple form.","ref":"VintageNet.html#t:any_ip_address/0"},{"type":"type","title":"VintageNet.configure_options/0","doc":"Valid options for `VintageNet.configure/3`\n\n* `:persist` - Whether or not to save the configuration (defaults to `true`)","ref":"VintageNet.html#t:configure_options/0"},{"type":"type","title":"VintageNet.connection_status/0","doc":"Interface connection status\n\n* `:disconnected` - The interface doesn't exist or it's not connected\n* `:lan` - The interface is connected to the LAN, but may not be able\n  reach the Internet\n* `:internet` - Packets going through the interface should be able to\n  reach the Internet","ref":"VintageNet.html#t:connection_status/0"},{"type":"type","title":"VintageNet.ifname/0","doc":"A name for the network interface\n\nNames depend on the device drivers and any software that may rename them.\nTypical names on Nerves are:\n\n* \"eth0\", \"eth1\", etc. for wired Ethernet interfaces\n* \"wlan0\", etc. for WiFi interfaces\n* \"ppp0\" for cellular modems\n* \"usb0\" for gadget USB virtual Ethernet interfaces","ref":"VintageNet.html#t:ifname/0"},{"type":"type","title":"VintageNet.info_options/0","doc":"Valid options for `VintageNet.info/1`\n\n* `:redact` - Whether to hide passwords and similar information from the output (defaults to `true`)","ref":"VintageNet.html#t:info_options/0"},{"type":"type","title":"VintageNet.interface_type/0","doc":"Interface type\n\nThis is a coarse characterization of a network interface that can be useful\nfor prioritizing interfaces.\n\n* `:ethernet` - Wired-based networking. Generally expected to be fast.\n* `:wifi` - Wireless networking. Expected to be not as fast as Ethernet,\n* `:mobile` - Cellular-based networking. Expected to be metered and slower\n  than `:wifi` and `:ethernet`\n* `:local` - Interfaces that never route to other hosts\n* `:unknown` - Catch-all when the network interface can't be categorized\n\nThese are general categories that are helpful for VintageNet's default\nrouting prioritization. See `VintageNet.Route.DefaultMetric` for more\ninformation on the use.","ref":"VintageNet.html#t:interface_type/0"},{"type":"type","title":"VintageNet.ipv4_prefix_length/0","doc":"The number of bits to use for an IPv4 subnet\n\nFor example, if you have a subnet mask of 255.255.255.0, then the prefix\nlength would be 24.","ref":"VintageNet.html#t:ipv4_prefix_length/0"},{"type":"type","title":"VintageNet.ipv6_prefix_length/0","doc":"The number of bits to use for an IPv6 subnet","ref":"VintageNet.html#t:ipv6_prefix_length/0"},{"type":"type","title":"VintageNet.pattern/0","doc":"A pattern for matching against VintageNet properties\n\nPatterns are used when subscribing for network property changes or getting a\nset of properties and their values.\n\nSince properties are organized hierarchically, the default way of matching patterns is to match on prefixes. It's also\npossible to use the `:_` wildcard to match anything at a position.","ref":"VintageNet.html#t:pattern/0"},{"type":"type","title":"VintageNet.prefix_length/0","doc":"The number of IP address bits for the subnet","ref":"VintageNet.html#t:prefix_length/0"},{"type":"type","title":"VintageNet.property/0","doc":"A VintageNet property\n\nVintageNet uses lists of strings to name networking configuration and status\nitems.","ref":"VintageNet.html#t:property/0"},{"type":"type","title":"VintageNet.value/0","doc":"A property's value\n\nSee the `README.md` for documentation on available properties.","ref":"VintageNet.html#t:value/0"},{"type":"module","title":"VintageNet.Connectivity.CheckLogic","doc":"Core logic for determining internet connectivity based on check results\n\nThis module is meant to be used by `InternetChecker` and others for\ndetermining when to run checks and how many failures should change the\nnetwork interface's state.\n\nIt implements a state machine that figures out what the connectivity status\nis based on internet-connectivity check successes and fails. It also returns\nhow long to wait between checks.\n\n```mermaid\nstateDiagram-v2\n  direction LR\n  [*]-->internet : init\n\n  state connected {\n    internet-->lan : max failures\n    lan-->internet : check succeeded\n  }\n  connected-->disconnected : ifdown\n  disconnected-->lan : ifup\n```","ref":"VintageNet.Connectivity.CheckLogic.html"},{"type":"function","title":"VintageNet.Connectivity.CheckLogic.check_failed/1","doc":"Call this when an Internet connectivity check fails\n\nDepending on how many failures have happened it a row, the connectivity may\nbe degraded to `:lan`.","ref":"VintageNet.Connectivity.CheckLogic.html#check_failed/1"},{"type":"function","title":"VintageNet.Connectivity.CheckLogic.check_succeeded/1","doc":"Call this when an Internet connectivity check succeeds","ref":"VintageNet.Connectivity.CheckLogic.html#check_succeeded/1"},{"type":"function","title":"VintageNet.Connectivity.CheckLogic.ifdown/1","doc":"Call this when the interface goes down\n\nThe interface will be categorized as `:disconnected` until `ifup/1` gets\ncalled again.","ref":"VintageNet.Connectivity.CheckLogic.html#ifdown/1"},{"type":"function","title":"VintageNet.Connectivity.CheckLogic.ifup/1","doc":"Call this when the interface comes up\n\nIt is assumed that the interface has LAN connectivity now and a check will\nbe scheduled to happen shortly.","ref":"VintageNet.Connectivity.CheckLogic.html#ifup/1"},{"type":"function","title":"VintageNet.Connectivity.CheckLogic.init/1","doc":"Initialize check state machine\n\nPass in the assumed connection status. This is a best guess to start things out.","ref":"VintageNet.Connectivity.CheckLogic.html#init/1"},{"type":"type","title":"VintageNet.Connectivity.CheckLogic.state/0","doc":"","ref":"VintageNet.Connectivity.CheckLogic.html#t:state/0"},{"type":"module","title":"VintageNet.Connectivity.Inspector","doc":"This module looks at the network activity of all TCP socket connections known\nto Erlang/OTP to deduce whether the internet is working.\n\nTo use it, call `check_internet/2`, save the returned cache, and then call it\nagain a minute later (or so). If any socket has transferred data in both\ndirections to an off-LAN host on the interface of interest, then it will\nreturn that the internet is available.\n\nThis has a couple advantages:\n\n1. No data is sent to perform the check which is good for metered connections\n2. Most long-lived TCP connections have a keepalive mechanism that generates\n   traffic, so this piggy-backs off that existing connectivity check.\n3. Devices can be behind very restrictive firewalls and internet connectivity\n   can still be verified without knowing which IP/port/protocol combinations\n   are allowed.\n\nIt is not perfect:\n\n1. It only works on long-lived TCP connections.\n2. The TCP connection must be sending and receiving data. If the keapalive is\n   longer than the `check_internet/2`\n3. It doesn't help if nobody is using the network interface.\n4. It may have scalability issues if there are a LOT of TCP sockets.","ref":"VintageNet.Connectivity.Inspector.html"},{"type":"function","title":"VintageNet.Connectivity.Inspector.check_internet/2","doc":"Check whether the internet is accessible on the specified interface\n\nPass an empty map for the cache parameter for the first call. Then pass it\nback the returned cache for each subsequent call. If any TCP socket that's\nconnected to a computer on another subnet and that's using the passed in\nnetwork interface has send AND received data since the previous call, then\n`:internet` is returned. If not, then usually `:unknown` is returned to\nsignify that internet may be available, but we just don't know. If the\ninterface doesn't have an IP address, then `:no_internet` is returned, since\nthat's a prerequisite to communicating with anyone on the internet.","ref":"VintageNet.Connectivity.Inspector.html#check_internet/2"},{"type":"function","title":"VintageNet.Connectivity.Inspector.on_interface?/2","doc":"Return true if an IP address is on one of the subnets in a list","ref":"VintageNet.Connectivity.Inspector.html#on_interface?/2"},{"type":"function","title":"VintageNet.Connectivity.Inspector.on_subnet?/2","doc":"Return true if an IP address is in the subnet","ref":"VintageNet.Connectivity.Inspector.html#on_subnet?/2"},{"type":"function","title":"Examples - VintageNet.Connectivity.Inspector.on_subnet?/2","doc":"iex> Inspector.on_subnet?({192,168,0,50}, {{192,168,0,1}, {255,255,255,0}})\n    true\n\n    iex> Inspector.on_subnet?({192,168,5,1}, {{192,168,0,1}, {255,255,255,0}})\n    false","ref":"VintageNet.Connectivity.Inspector.html#on_subnet?/2-examples"},{"type":"function","title":"VintageNet.Connectivity.Inspector.routed_address?/2","doc":"Returns true if the specified address is not on the network interface\n\nThis function is useful for checking whether an address is on the Internet if\nyou don't trust the DNS server. Captive portals, for example, can give back\nIP addresses that are local. It's not guaranteed, but it would be pointless to\ncheck those IP's if you're looking for the Internet.","ref":"VintageNet.Connectivity.Inspector.html#routed_address?/2"},{"type":"type","title":"VintageNet.Connectivity.Inspector.cache/0","doc":"Cache for use between `check_internet/2` calls. Initialize to an empty map.","ref":"VintageNet.Connectivity.Inspector.html#t:cache/0"},{"type":"type","title":"VintageNet.Connectivity.Inspector.result/0","doc":"The return tuple for `check_internet/2`\n\n* `:internet` - at least one TCP connection sent and received data to a\n  non-LAN IP address\n* `:unknown` - no conclusion could be made\n* `:no_internet` - the interface didn't have an IP address, so Internet is\n  definitely not available\n\nSave the cache away and pass it to the next call to `check_internet/2`.","ref":"VintageNet.Connectivity.Inspector.html#t:result/0"},{"type":"type","title":"VintageNet.Connectivity.Inspector.status/0","doc":"Internet connectivity status\n\n  * `:internet` - the internet is available\n  * `:unknown` - not sure\n  * `:no_internet` - the internet is definitely not available","ref":"VintageNet.Connectivity.Inspector.html#t:status/0"},{"type":"module","title":"VintageNet.Connectivity.InternetChecker","doc":"This GenServer monitors a network interface for Internet connectivity\n\nInternet connectivity is determined by reachability to an IP address.\nIf that address is reachable then other this updates a property to\nreflect that. Otherwise, the network interface is assumed to merely\nhave LAN connectivity if it's up.","ref":"VintageNet.Connectivity.InternetChecker.html"},{"type":"function","title":"VintageNet.Connectivity.InternetChecker.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"VintageNet.Connectivity.InternetChecker.html#child_spec/1"},{"type":"function","title":"VintageNet.Connectivity.InternetChecker.start_link/1","doc":"Start the connectivity checker GenServer","ref":"VintageNet.Connectivity.InternetChecker.html#start_link/1"},{"type":"module","title":"VintageNet.Connectivity.LANChecker","doc":"This GenServer monitors a network interface for LAN connectivity\n\nCurrently LAN connectivity simply looks to see if it's possible to\nsend a packet on the interface. It might or might not get to the\ndesired destination on the LAN, but it won't obviously fail.\n\nThis is an alternative to the InternetConnectivityChecker that\nactively monitors reachability to a host.","ref":"VintageNet.Connectivity.LANChecker.html"},{"type":"function","title":"VintageNet.Connectivity.LANChecker.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"VintageNet.Connectivity.LANChecker.html#child_spec/1"},{"type":"function","title":"VintageNet.Connectivity.LANChecker.start_link/1","doc":"Start the connectivity checker GenServer","ref":"VintageNet.Connectivity.LANChecker.html#start_link/1"},{"type":"module","title":"VintageNet.Connectivity.TCPPing","doc":"Test connectivity by making a connection using TCP\n\nConnectivity with a remote host can be checked by making a TCP connection to\nit. The connection either works, the connection is refused, or it times out.\nThe first two cases indicate connectivity.\n\nNormally ICMP is used for testing connectivity, but that requires the new\nsocket API and a Linux kernel with `net.ipv4.ping_group_range` enabled.  This\nway usually works unless a device is behind a strict firewall, but there's\nusually at least one IP address/port on the Internet that they allow.","ref":"VintageNet.Connectivity.TCPPing.html"},{"type":"function","title":"VintageNet.Connectivity.TCPPing.ping/2","doc":"Check connectivity with another device\n\nThe \"ping\" is really a TCP connection attempt from the specified interface to\nan IP address and port. Failures to connect don't necessarily mean that the\nInternet is down, but it's likely especially if the server that's specified\nin the configuration is highly available.\n\nSource IP-based routing is required for the TCP connect to go out the right\nnetwork interface. This is configured by default when using VintageNet.","ref":"VintageNet.Connectivity.TCPPing.html#ping/2"},{"type":"type","title":"VintageNet.Connectivity.TCPPing.ping_error_reason/0","doc":"","ref":"VintageNet.Connectivity.TCPPing.html#t:ping_error_reason/0"},{"type":"module","title":"VintageNet.DHCP.Options","doc":"DHCP Options","ref":"VintageNet.DHCP.Options.html"},{"type":"type","title":"VintageNet.DHCP.Options.t/0","doc":"A map of options and other information reported by udhcpc\n\nHere's an example:\n\n```elixir\n%{\n  broadcast: {192, 168, 7, 255},\n  dns: {192, 168, 7, 1},\n  domain: \"hunleth.lan\",\n  hostname: \"nerves-9780\",\n  ip: {192, 168, 7, 190},\n  lease: 86400,\n  mask: 24,\n  router: {192, 168, 7, 1},\n  serverid: {192, 168, 7, 1},\n  siaddr: {192, 168, 7, 1},\n  subnet: {255, 255, 255, 0}\n}\n```","ref":"VintageNet.DHCP.Options.html#t:t/0"},{"type":"module","title":"VintageNet.IP","doc":"This module contains utilities for handling IP addresses.\n\nBy far the most important part of handling IP addresses is to\npay attention to whether your addresses are names, IP addresses\nas strings or IP addresses at tuples. This module doesn't resolve\nnames. While IP addresses in string form are convenient to type,\nnearly all Erlang and Elixir code uses IP addresses in tuple\nform.","ref":"VintageNet.IP.html"},{"type":"function","title":"VintageNet.IP.cidr_to_string/2","doc":"Convert an IP address w/ prefix to a CIDR-formatted string\n\nExamples:\n\n    iex> VintageNet.IP.cidr_to_string({192, 168, 0, 1}, 24)\n    \"192.168.0.1/24\"","ref":"VintageNet.IP.html#cidr_to_string/2"},{"type":"function","title":"VintageNet.IP.ip_to_string/1","doc":"Convert an IP address to a string\n\nExamples:\n\n    iex> VintageNet.IP.ip_to_string({192, 168, 0, 1})\n    \"192.168.0.1\"\n\n    iex> VintageNet.IP.ip_to_string(\"192.168.9.1\")\n    \"192.168.9.1\"\n\n    iex> VintageNet.IP.ip_to_string({65152, 0, 0, 0, 0, 0, 0, 1})\n    \"fe80::1\"","ref":"VintageNet.IP.html#ip_to_string/1"},{"type":"function","title":"VintageNet.IP.ip_to_tuple/1","doc":"Convert an IP address to tuple form\n\nExamples:\n\n    iex> VintageNet.IP.ip_to_tuple(\"192.168.0.1\")\n    {:ok, {192, 168, 0, 1}}\n\n    iex> VintageNet.IP.ip_to_tuple({192, 168, 1, 1})\n    {:ok, {192, 168, 1, 1}}\n\n    iex> VintageNet.IP.ip_to_tuple(\"fe80::1\")\n    {:ok, {65152, 0, 0, 0, 0, 0, 0, 1}}\n\n    iex> VintageNet.IP.ip_to_tuple({65152, 0, 0, 0, 0, 0, 0, 1})\n    {:ok, {65152, 0, 0, 0, 0, 0, 0, 1}}\n\n    iex> VintageNet.IP.ip_to_tuple(\"bologna\")\n    {:error, \"Invalid IP address: bologna\"}","ref":"VintageNet.IP.html#ip_to_tuple/1"},{"type":"function","title":"VintageNet.IP.ip_to_tuple!/1","doc":"Raising version of ip_to_tuple/1","ref":"VintageNet.IP.html#ip_to_tuple!/1"},{"type":"function","title":"VintageNet.IP.ipv4_broadcast_address/2","doc":"Return the IPv4 broadcast address for the specified subnet and prefix\n\nExamples:\n\n    iex> VintageNet.IP.ipv4_broadcast_address({192, 168, 1, 50}, 24)\n    {192, 168, 1, 255}\n\n    iex> VintageNet.IP.ipv4_broadcast_address({74, 125, 227, 0}, 29)\n    {74, 125, 227, 7}","ref":"VintageNet.IP.html#ipv4_broadcast_address/2"},{"type":"function","title":"VintageNet.IP.prefix_length_to_subnet_mask/2","doc":"Convert an IPv4 or IPv6 prefix length to a subnet mask.\n\nExamples:\n\n    iex> VintageNet.IP.prefix_length_to_subnet_mask(:inet, 24)\n    {255, 255, 255, 0}\n\n    iex> VintageNet.IP.prefix_length_to_subnet_mask(:inet, 28)\n    {255, 255, 255, 240}\n\n    iex> VintageNet.IP.prefix_length_to_subnet_mask(:inet6, 64)\n    {65535, 65535, 65535, 65535, 0, 0, 0, 0}","ref":"VintageNet.IP.html#prefix_length_to_subnet_mask/2"},{"type":"function","title":"VintageNet.IP.subnet_mask_to_prefix_length/1","doc":"Convert an IPv4 subnet mask to a prefix length.\n\nExamples:\n\n    iex> VintageNet.IP.subnet_mask_to_prefix_length({255, 255, 255, 0})\n    {:ok, 24}\n\n    iex> VintageNet.IP.subnet_mask_to_prefix_length({192, 168, 1, 1})\n    {:error, \"{192, 168, 1, 1} is not a valid IPv4 subnet mask\"}","ref":"VintageNet.IP.html#subnet_mask_to_prefix_length/1"},{"type":"function","title":"VintageNet.IP.to_subnet/2","doc":"Utility function to trim an IP address to its subnet\n\nExamples:\n\n    iex> VintageNet.IP.to_subnet({192, 168, 1, 50}, 24)\n    {192, 168, 1, 0}\n\n    iex> VintageNet.IP.to_subnet({192, 168, 255, 50}, 22)\n    {192, 168, 252, 0}\n\n    iex> VintageNet.IP.to_subnet({64768, 43690, 0, 0, 4144, 58623, 65276, 33158}, 64)\n    {64768, 43690, 0, 0, 0, 0, 0, 0}","ref":"VintageNet.IP.html#to_subnet/2"},{"type":"module","title":"VintageNet.IP.DhcpdConfig","doc":"This is a helper module for VintageNet.Technology implementations that use\na DHCP server.\n\nDHCP server parameters are:\n\n* `:start` - Start of the lease block\n* `:end` - End of the lease block\n* `:max_leases` - The maximum number of leases\n* `:decline_time` - The amount of time that an IP will be reserved (leased to nobody)\n* `:conflict_time` -The amount of time that an IP will be reserved\n* `:offer_time` - How long an offered address is reserved (seconds)\n* `:min_lease` - If client asks for lease below this value, it will be rounded up to this value (seconds)\n* `:auto_time` - The time period at which udhcpd will write out leases file.\n* `:static_leases` - list of `{mac_address, ip_address}`\n* `:options` - a map DHCP response options to set. Such as:\n  * `:dns` - IP_LIST\n  * `:domain` -  STRING - [0x0f] client's domain suffix\n  * `:hostname` - STRING\n  * `:mtu` - NUM\n  * `:router` - IP_LIST\n  * `:search` - STRING_LIST - [0x77] search domains\n  * `:serverid` - IP (defaults to the interface's IP address)\n  * `:subnet` or `:netmask` - IP as a subnet mask (`:netmask` is an alias for `:subnet`)\n\n> #### :options {: .info}\n> Options may also be passed in as integers. These are passed directly to the DHCP server\n> and their values are strings that are not interpreted by VintageNet. Use this to support\n> custom DHCP header options. For more details on DHCP response options see RFC 2132","ref":"VintageNet.IP.DhcpdConfig.html"},{"type":"module","title":"Example - VintageNet.IP.DhcpdConfig","doc":"```\n  VintageNet.configure(\"wlan0\", %{\n    type: VintageNetWiFi,\n    vintage_net_wifi: %{\n      networks: [\n        %{\n          mode: :ap,\n          ssid: \"test ssid\",\n          key_mgmt: :none\n        }\n      ]\n    },\n    dhcpd: %{\n      start: \"192.168.24.2\",\n      end: \"192.168.24.10\",\n      options: %{\n        dns: [\"1.1.1.1\", \"1.0.0.1\"],\n        netmask: \"255.255.255.0\",\n        router: [\"192.168.24.1\"]\n      }\n    }\n  })\n```","ref":"VintageNet.IP.DhcpdConfig.html#module-example"},{"type":"function","title":"VintageNet.IP.DhcpdConfig.add_config/3","doc":"Add udhcpd configuration commands for running a DHCP server","ref":"VintageNet.IP.DhcpdConfig.html#add_config/3"},{"type":"function","title":"VintageNet.IP.DhcpdConfig.normalize/1","doc":"Normalize the DHCPD parameters in a configuration.","ref":"VintageNet.IP.DhcpdConfig.html#normalize/1"},{"type":"module","title":"VintageNet.IP.DnsdConfig","doc":"This is a helper module for VintageNet.Technology implementations that use\nthe Busybox DNS server.\n\nDNS functionality is only supported for IPv4 configurations using static IP\naddresses.\n\nDNS server parameters are:\n\n* `:port` - The port to use (defaults to 53)\n* `:ttl` - DNS record TTL in seconds (defaults to 120)\n* `:records` - DNS A records (required)\n\nThe `:records` option is a list of name/IP address tuples. For example:\n\n```\n[{\"example.com\", {1, 2, 3, 4}}]\n```\n\nOnly IPv4 addresses are supported. Addresses may be specified as strings or\ntuples, but will be normalized to tuple form before being applied.","ref":"VintageNet.IP.DnsdConfig.html"},{"type":"function","title":"VintageNet.IP.DnsdConfig.add_config/3","doc":"Add dnsd configuration commands for running a DNSD server","ref":"VintageNet.IP.DnsdConfig.html#add_config/3"},{"type":"function","title":"VintageNet.IP.DnsdConfig.normalize/1","doc":"Normalize the DNSD parameters in a configuration.","ref":"VintageNet.IP.DnsdConfig.html#normalize/1"},{"type":"module","title":"VintageNet.IP.IPv4Config","doc":"This is a helper module for VintageNet.Technology implementations that use\nIPv4.\n\nIPv4 configuration is specified under the `:ipv4` key in the configuration map.\nFields include:\n\n* `:method` - `:dhcp`, `:static`, or `:disabled`\n\nThe `:dhcp` method currently has no additional fields.\n\nThe `:static` method uses the following fields:\n\n* `:address` - the IP address\n* `:prefix_length` - the number of bits in the IP address to use for the subnet (e.g., 24)\n* `:netmask` - either this or `prefix_length` is used to determine the subnet. If you\n  have a choice, use `prefix_length`\n* `:gateway` - the default gateway for this interface (optional)\n* `:name_servers` - a list of DNS servers (optional)\n* `:domain` - DNS search domain (optional)\n\nConfiguration normalization converts `:netmask` to `:prefix_length`.","ref":"VintageNet.IP.IPv4Config.html"},{"type":"function","title":"VintageNet.IP.IPv4Config.add_config/3","doc":"Add IPv4 configuration commands for supporting static and dynamic IP addressing","ref":"VintageNet.IP.IPv4Config.html#add_config/3"},{"type":"function","title":"VintageNet.IP.IPv4Config.normalize/1","doc":"Normalize the IPv4 parameters in a configuration.","ref":"VintageNet.IP.IPv4Config.html#normalize/1"},{"type":"module","title":"VintageNet.Interface","doc":"Manage a network interface at a very high level\n\nThis module handles configuring network interfaces, making sure that configuration failures\nget retried, and then cleaning up after it's not needed.\n\nThe actual code that supplies the configuration implements the `VintageNet.Technology`\nbehaviour.","ref":"VintageNet.Interface.html"},{"type":"function","title":"VintageNet.Interface.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"VintageNet.Interface.html#child_spec/1"},{"type":"function","title":"VintageNet.Interface.configure/3","doc":"Set a configuration on an interface\n\nConfigurations with invalid parameters raise exceptions. It's\nstill possible that network configurations won't work even if they\ndon't raise, but it should be due to something in the environment.\nFor example, a network cable isn't plugged in or a WiFi access point\nis out of range.","ref":"VintageNet.Interface.html#configure/3"},{"type":"function","title":"VintageNet.Interface.deconfigure/2","doc":"Deconfigure the interface\n\nThis doesn't exit this GenServer, but the interface\nwon't be usable in any real way until it's configured\nagain.\n\nThis function is not normally called.","ref":"VintageNet.Interface.html#deconfigure/2"},{"type":"function","title":"VintageNet.Interface.ioctl/3","doc":"Run an I/O command on the specified interface","ref":"VintageNet.Interface.html#ioctl/3"},{"type":"function","title":"VintageNet.Interface.start_link/1","doc":"Start up an interface\n\nParameters:\n\n* `ifname` - which interface","ref":"VintageNet.Interface.html#start_link/1"},{"type":"function","title":"VintageNet.Interface.stop/1","doc":"Stop the interface\n\nNote that this doesn't deconfigure it.","ref":"VintageNet.Interface.html#stop/1"},{"type":"function","title":"VintageNet.Interface.to_raw_config/2","doc":"Convert a configuration to a raw one\n\nThis can be used to validate a configuration without applying it.","ref":"VintageNet.Interface.html#to_raw_config/2"},{"type":"function","title":"VintageNet.Interface.wait_until_configured/1","doc":"Wait for the interface to be configured","ref":"VintageNet.Interface.html#wait_until_configured/1"},{"type":"module","title":"VintageNet.Interface.CommandRunner","doc":"The CommandRunner module runs commands specified in RawConfigs\n\nSee the `RawConfig` documentation for where lists of commands\nare specified. The following commands are supported:\n\n* `{:run, command, args}` - Run a system command\n* `{:run_ignore_exit, command, args}` - Same as `:run`, but without the exit status check\n* `{:fun, module, function_name, args}` - Run a function by MFArgs\n* `{:fun, fun}` - Run a function. Using the MFArgs form is preferred since it's\n                  easier to verify in unit tests.\n\nCommandRunner also implements RawConfig's file creation and\ncleanup logic.","ref":"VintageNet.Interface.CommandRunner.html"},{"type":"function","title":"VintageNet.Interface.CommandRunner.create_files/1","doc":"Create a list of files","ref":"VintageNet.Interface.CommandRunner.html#create_files/1"},{"type":"function","title":"VintageNet.Interface.CommandRunner.remove_files/1","doc":"Remove a list of files","ref":"VintageNet.Interface.CommandRunner.html#remove_files/1"},{"type":"function","title":"VintageNet.Interface.CommandRunner.run/1","doc":"Run one or more commands\n\nSee the module docs for documentation on commands.","ref":"VintageNet.Interface.CommandRunner.html#run/1"},{"type":"module","title":"VintageNet.Interface.EAPStatus","doc":"Status of an EAP connection.","ref":"VintageNet.Interface.EAPStatus.html"},{"type":"module","title":"Keys - VintageNet.Interface.EAPStatus","doc":"* `status` Status of the connection.\n  * `:started` - the AP was assosiated and EAP is started.\n  * `:success` - the EAP connection was successful\n  * `:failure` - the EAP connection failed.\n* `method` - EAP method used to authenticate. See the typespec for available values.\n* `timestamp` - DateTime of the most recent EAP event.\n* `remote_certificate_verified?` - if the cert was verified by the EAP server.","ref":"VintageNet.Interface.EAPStatus.html#module-keys"},{"type":"type","title":"VintageNet.Interface.EAPStatus.method/0","doc":"Can be one of: as defined in `eap_defs.h` in the hostapd source.\nNONE\nIDENTITY\nNOTIFICATION\nNAK\nMD5\nOTP\nGTC\nTLS\nLEAP\nSIM\nTTLS\nAKA\nPEAP\nMSCHAPV2\nTLV\nTNC\nFAST\nPAX\nPSK\nSAKE\nIKEV2\nAKA_PRIME\nGPSK\nPWD\nEKE\nTEAP\nEXPANDED","ref":"VintageNet.Interface.EAPStatus.html#t:method/0"},{"type":"type","title":"VintageNet.Interface.EAPStatus.t/0","doc":"","ref":"VintageNet.Interface.EAPStatus.html#t:t/0"},{"type":"module","title":"VintageNet.Interface.IfupDaemon","doc":"Wrap MuonTrap.Daemon to start and stop a program based on whether the network is up\n\nUnlike MuonTrap.Daemon, the arguments are called out in the child_spec so it looks like\nthis:\n\n```\n{VintageNet.Interface.IfupDaemon, ifname: ifname, command: program, args: arguments, opts: options]}\n```","ref":"VintageNet.Interface.IfupDaemon.html"},{"type":"function","title":"VintageNet.Interface.IfupDaemon.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"VintageNet.Interface.IfupDaemon.html#child_spec/1"},{"type":"function","title":"VintageNet.Interface.IfupDaemon.running?/1","doc":"Return whether the daemon is running","ref":"VintageNet.Interface.IfupDaemon.html#running?/1"},{"type":"function","title":"VintageNet.Interface.IfupDaemon.start_link/1","doc":"Start the IfupDaemon","ref":"VintageNet.Interface.IfupDaemon.html#start_link/1"},{"type":"module","title":"VintageNet.Interface.NameUtilities","doc":"Module for classifying network interfaces","ref":"VintageNet.Interface.NameUtilities.html"},{"type":"function","title":"VintageNet.Interface.NameUtilities.get_instance/1","doc":"Extract a number out of an interface name\n\nThe result is the interface index for most interfaces seen\non Nerves (eth0, eth1, ...), and something quite imperfect when using predictable\ninterface naming (enp6s0, enp6s1).\n\nThis is currently used to order priorities when there are two\ninterfaces available of the same type that cannot be differentiated\nby other means. It has the one property of being easy to explain.","ref":"VintageNet.Interface.NameUtilities.html#get_instance/1"},{"type":"function","title":"VintageNet.Interface.NameUtilities.to_type/1","doc":"Classify a network type based on its name\n\nExamples\n\n    iex> NameUtilities.to_type(\"eth0\")\n    :ethernet\n\n    iex> NameUtilities.to_type(\"wlp5s0\")\n    :wifi\n\n    iex> NameUtilities.to_type(\"wwan0\")\n    :mobile","ref":"VintageNet.Interface.NameUtilities.html#to_type/1"},{"type":"module","title":"VintageNet.Interface.RawConfig","doc":"Raw configuration for an interface\n\nThis struct contains the low-level instructions for how to configure and\ndeconfigure an interface.\n\nFields:\n\n* `:ifname` - the name of the interface (e.g., `\"eth0\"`)\n* `:type` - the type of network interface (aka the module that created the config)\n* `:source_config` - the configuration that generated this one\n* `:required_ifnames` - a list of ifnames that need to exist before starting this configuration. (e.g. `[\"eth0\"]`)\n* `:retry_millis` - if bringing the interface up fails, wait this amount of time before retrying\n* `:files` - a list of file path, content tuples\n* `:restart_strategy` - the restart strategy for the list of `child_specs`. I.e., `:one_for_one | :one_for_all | :rest_for_one`\n* `:child_specs` - a set of child_specs for GenServers to start up and supervise\n* `:up_cmd_millis` - the maximum amount of time to allow the up command list to take\n* `:up_cmds` - a list of commands to run to configure the interface\n* `:down_cmd_millis` - the maximum amount of time to allow the down command list to take\n* `:down_cmds` - a list of commands to run to bring the interface down\n* `:cleanup_files` - additional files to delete (the files listed in `files` are deleted too)","ref":"VintageNet.Interface.RawConfig.html"},{"type":"type","title":"VintageNet.Interface.RawConfig.command/0","doc":"A command to run for RawConfig.up_cmds or RawConfig.down_cmds","ref":"VintageNet.Interface.RawConfig.html#t:command/0"},{"type":"type","title":"VintageNet.Interface.RawConfig.file_contents/0","doc":"","ref":"VintageNet.Interface.RawConfig.html#t:file_contents/0"},{"type":"type","title":"VintageNet.Interface.RawConfig.t/0","doc":"","ref":"VintageNet.Interface.RawConfig.html#t:t/0"},{"type":"behaviour","title":"VintageNet.InterfaceRenamer","doc":"Wrapper around the `ip` command for renaming interfaces","ref":"VintageNet.InterfaceRenamer.html"},{"type":"function","title":"VintageNet.InterfaceRenamer.rename/2","doc":"Renames an interface","ref":"VintageNet.InterfaceRenamer.html#rename/2"},{"type":"callback","title":"VintageNet.InterfaceRenamer.rename_interface/2","doc":"","ref":"VintageNet.InterfaceRenamer.html#c:rename_interface/2"},{"type":"module","title":"VintageNet.InterfacesMonitor","doc":"Monitor available interfaces\n\nCurrently this works by polling the system for what interfaces are visible.\nThey may or may not be configured.","ref":"VintageNet.InterfacesMonitor.html"},{"type":"function","title":"VintageNet.InterfacesMonitor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"VintageNet.InterfacesMonitor.html#child_spec/1"},{"type":"function","title":"VintageNet.InterfacesMonitor.force_clear_ipv4_addresses/1","doc":"Force clear all addresses\n\nThis is useful to notify everyone that an address should not be used\nimmediately. This can be used to fix a race condition where the blip\nfor an address going away to coming back isn't reported.","ref":"VintageNet.InterfacesMonitor.html#force_clear_ipv4_addresses/1"},{"type":"function","title":"VintageNet.InterfacesMonitor.start_link/1","doc":"","ref":"VintageNet.InterfacesMonitor.html#start_link/1"},{"type":"module","title":"VintageNet.NameResolver","doc":"This module manages the contents of \"/etc/resolv.conf\".\n\nThis file is used by the C standard library and by Erlang for resolving\ndomain names.  Since both C programs and Erlang can do resolution, debugging\nproblems in this area can be confusing due to varying behavior based on who's\nresolving at the time. See the `/etc/erl_inetrc` file on the target to review\nErlang's configuration.\n\nThis module assumes exclusive ownership on \"/etc/resolv.conf\", so if any\nother code in the system tries to modify the file, their changes will be lost\non the next update.\n\nIt is expected that each network interface provides a configuration. This\nmodule will track configurations to network interfaces so that it can reflect\nwhich resolvers are around. Resolver order isn't handled.","ref":"VintageNet.NameResolver.html"},{"type":"function","title":"VintageNet.NameResolver.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"VintageNet.NameResolver.html#child_spec/1"},{"type":"function","title":"VintageNet.NameResolver.clear/1","doc":"Clear all entries in \"/etc/resolv.conf\" that are associated with\nthe specified interface.","ref":"VintageNet.NameResolver.html#clear/1"},{"type":"function","title":"VintageNet.NameResolver.clear_all/0","doc":"Completely clear out \"/etc/resolv.conf\".","ref":"VintageNet.NameResolver.html#clear_all/0"},{"type":"function","title":"VintageNet.NameResolver.setup/3","doc":"Set the search domain and name server list for the specified interface.\n\nThis replaces any entries in the `/etc/resolv.conf` for this interface.","ref":"VintageNet.NameResolver.html#setup/3"},{"type":"function","title":"VintageNet.NameResolver.start_link/1","doc":"Start the resolv.conf manager.\n\nAccepted args:\n\n* `resolvconf` - path to the resolvconf file\n* `additional_name_servers` - list of additional servers","ref":"VintageNet.NameResolver.html#start_link/1"},{"type":"function","title":"VintageNet.NameResolver.stop/0","doc":"Stop the resolv.conf manager.","ref":"VintageNet.NameResolver.html#stop/0"},{"type":"type","title":"VintageNet.NameResolver.state/0","doc":"","ref":"VintageNet.NameResolver.html#t:state/0"},{"type":"behaviour","title":"VintageNet.Persistence","doc":"Customize the way VintageNet saves and loads configurations","ref":"VintageNet.Persistence.html"},{"type":"function","title":"VintageNet.Persistence.call/2","doc":"","ref":"VintageNet.Persistence.html#call/2"},{"type":"callback","title":"VintageNet.Persistence.clear/1","doc":"Clear out a previously saved configuration","ref":"VintageNet.Persistence.html#c:clear/1"},{"type":"callback","title":"VintageNet.Persistence.enumerate/0","doc":"Enumerate the interfaces that have saved configurations\n\nThis returns a list of interface names.","ref":"VintageNet.Persistence.html#c:enumerate/0"},{"type":"callback","title":"VintageNet.Persistence.load/1","doc":"Load the configuration of an interface","ref":"VintageNet.Persistence.html#c:load/1"},{"type":"callback","title":"VintageNet.Persistence.save/2","doc":"Save the configuration for the specified interface","ref":"VintageNet.Persistence.html#c:save/2"},{"type":"module","title":"VintageNet.Persistence.FlatFile","doc":"Save and load configurations from flat files","ref":"VintageNet.Persistence.FlatFile.html"},{"type":"module","title":"VintageNet.Persistence.Null","doc":"Don't save or load configuration at all.","ref":"VintageNet.Persistence.Null.html"},{"type":"behaviour","title":"VintageNet.PowerManager","doc":"This is a behaviour for implementing platform-specific power management.\n\nFrom VintageNet's point of view, network devices have the following\nlifecycle:\n\n```\noff --->  on ---> powering-off ---> off\n```\n\nPower management does not necessarily mean controlling the power. The end\neffect should be similar, since VintageNet will try to toggle the power off\nand on if the network interface doesn't seem to be working. For example,\nunloading the kernel module for the network device on \"power off\" and loading\nit on \"power on\" may have the desired effect of getting a network interface\nunstuck.\n\nWhen a device is \"on\", VintageNet expects to be regularly told that the\ndevice is working ok. Working ok is device dependent, but could be something\nlike the device has transmitted and received data. If VintageNet is not told\nthat the device is working for a long enough time, it will reset the device\nby powering it off and then back on again.\n\nVintageNet calls functions here based on how it wants to transition a device.\nVintageNet maintains the device's power status internally, so implementations\ncan blindly do what VintageNet tells them too in most cases. Powering on and\noff can be asynchronous to these function calls. VintageNet uses the presence\nof the networking interface (like \"wlan0\") to determine when the device is\nreally available for networking.\n\nThe following timeouts are important to consider (in milliseconds):\n\n1. `time_to_power_off`\n2. `power_on_hold_time`\n3. `min_power_off_time`\n4. `watchdog_timeout`\n\nThe `time_to_power_off` specifies the time in the `powering-off` state. This\nis the maximum time to allow for a graceful shutdown. VintageNet won't bother\nthe device until that time has expired. That means that if there's a request\nto use the device, it will wait the `powering-off` time before calling\n`finish_power_off` and then it will power the device back on. Device app\nnotes may have recommendations for this time.\n\nThe `power_on_hold_time` specifies how much time a device should be in the\n`powered-on` state before it is ok to power off again. This allows devices\nsome time to initialize and recover on their own.\n\nThe `min_power_off_time` specifies how long the device should remain powered\noff before it is powered back on.\n\nFinally, `watchdog_timeout` specifies how long to wait between notifications\nthat the device is ok. Code reports that a device is ok by calling\n`VintageNet.PowerManager.PMControl.pet_watchdog/1`.\n\nWhile normal Erlang supervision expects that it can restart processes\nimmediately and without regard to how long they have been running, bad things\ncan happen to hardware if too aggressively restarted. Devices also initialize\nasynchronously so it's hard to know when they're fully available and some\nflakiness may be naturally due to VintageNet not knowing how to wait for a\ncomponent to finish initialization. Please review your network device's power\nmanagement guidelines before too aggressively reducing hold times. Cellular\ndevices, in particular, want to signal their disconnection from the network\nto the tower and flush any unsaved configuration changes to Flash before\npower removal.\n\nHere's an example for a cellular device with a reset line connected to it:\n\n* `power_on` - De-assert the reset line. Return a `power_on_hold_time` of 10\n  minutes\n* `start_powering_off` - Open the UART and send the power down command to the\n  modem. Return a `time_to_power_off` of 1 minute.\n* `power_off` - Assert the reset line and return that power shouldn't be turned\n  back on for another 10 seconds.\n\nPowerManager implementation lifetimes are the same as VintageNet's. In other\nwords, they start and end with VintageNet. This is unlike a network interface\nwhich runs only as its existence and configuration allow. As such, VintageNet\nneeds to know about all PowerManager implementations in its application\nenvironment.  For example, add something like this to your `config.exs`:\n\n```elixir\nconfig :vintage_net,\n  power_managers: [{MyCellularPM, [ifname: \"ppp0\", watchdog_timeout: 60_000, reset_gpio: 123]}]\n```\n\nEach tuple is the implementation's module name and init arguments. VintageNet\nrequires `:ifname` to be set. If you're managing the power for an interface\nwith a dynamic name, enable predictable interface naming with `VintageNet`\nand use that name. The `watchdog_timeout` parameter is optional and defaults\nto one minute.","ref":"VintageNet.PowerManager.html"},{"type":"callback","title":"VintageNet.PowerManager.handle_info/2","doc":"Handle other messages\n\nAll unknown messages sent to the power management `GenServer` come here. This\ncallback is similar to `c:GenServer.handle_info/2`.\n\nTo receive your own messages here, send them to `self()` in code run in any\nof the other callbacks. Another option is to call\n`VintageNet.PowerManager.PMControl.send_message/2`","ref":"VintageNet.PowerManager.html#c:handle_info/2"},{"type":"callback","title":"VintageNet.PowerManager.init/1","doc":"Initialize state for managing the power to the specified interface\n\nThis is called on start and if the power management GenServer restarts. It\nshould not assume that hardware is powered down.\n\nIMPORTANT: VintageNet assumes that `init/1` runs quickly and succeeds. Errors\nand exceptions from calling `init/1` are handled by disabling the PowerManager.\nThe reason is that VintageNet has no knowledge on how to recover and disabling\na power manager was deemed less bad that having supervision tree failures\npropagate upwards to terminate VintageNet. Messages are logged if this does\nhappen.","ref":"VintageNet.PowerManager.html#c:init/1"},{"type":"callback","title":"VintageNet.PowerManager.power_off/1","doc":"Power off the hardware\n\nThis function should finish powering off the network interface hardware. Since\nthis is called after the graceful power down should have completed, it should\nforcefully turn off the power to the hardware.\n\nThe implementation also returns a time that power must remain off, in milliseconds.\n`power_on/1` won't be called until that time expires.","ref":"VintageNet.PowerManager.html#c:power_off/1"},{"type":"callback","title":"VintageNet.PowerManager.power_on/1","doc":"Power on the hardware for a network interface\n\nThe function should turn on power rails, deassert reset lines, load kernel\nmodules or do whatever else is necessary to make the interface show up in\nLinux.\n\nFailure handling is not supported by VintageNet yet, so if power up can fail\nand the right handling for that is to try again later, then this function\nshould do that.\n\nIt is ok for this function to return immediately. When the network interface\nappears, VintageNet will start trying to use it.\n\nThe return tuple should include the number of milliseconds VintageNet should\nwait before trying to power down the module again. This value should be\nsufficiently large to avoid getting into loops where VintageNet gives up on a\nnetwork interface before it has initialized. 10 minutes (600,000 milliseconds),\nfor example, is a reasonable setting.","ref":"VintageNet.PowerManager.html#c:power_on/1"},{"type":"callback","title":"VintageNet.PowerManager.start_powering_off/1","doc":"Start powering off the hardware for a network interface\n\nThis function should start a graceful shutdown of the network interface\nhardware.  It may return immediately. The return value specifies how long in\nmilliseconds VintageNet should wait before calling `power_off/2`. The idea is\nthat a graceful power off should be allowed some time to complete, but not\nforever.","ref":"VintageNet.PowerManager.html#c:start_powering_off/1"},{"type":"module","title":"VintageNet.PowerManager.PMControl","doc":"Power management control GenServer\n\nThis GenServer runs a PowerManager implementation for a network device. It\nprovides the API for powering on and off a device and for signally that it's\nworking well.\n\nInternally, it runs a state machine that translates power on and off requests\ninto actions sent to the `PowerManager` implementation. The state machine\nhandles the timing of actions so that hardware gets a chance to initialize\nand isn't reset to quickly. `PowerManager` implementations specify times.\n\nSince networking devices can sometimes hang or fail in unexpected ways, this\nmodule can power them off and on to try to get them back in a good state.\nThis is implemented in terms of a watchdog. Code that can detect the network\ndevice being in a good state should call `pet_watchdog/1`. For example, code\nthat checks internet connectivity could call `pet_watchdog/1` since that's\na pretty good sign that the device works. Other checks are possible. If\n`pet_watchdog/1` isn't called, this module will restart the network device.","ref":"VintageNet.PowerManager.PMControl.html"},{"type":"function","title":"VintageNet.PowerManager.PMControl.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"VintageNet.PowerManager.PMControl.html#child_spec/1"},{"type":"function","title":"VintageNet.PowerManager.PMControl.force_reset/1","doc":"Force reset\n\nThis is intended to be called based on human interaction. For example,\nby a UI button or by a developer who knows or strongly suspects that\nsomething is wrong with the network device it needs a reboot.\n\nResetting devices that have been powered off will NOT power them on.\n\nCalling this automatically is not recommended especially if it is used\nas an alternative to the watchdog mechanism. The reason is that it is\neasier to identify where the device is working than it is to identify\nevery way it can fail. Also, force reset ignores minimum on times\nsince assumption is that if someone wants to reset, they're ready\nto reset now.","ref":"VintageNet.PowerManager.PMControl.html#force_reset/1"},{"type":"function","title":"VintageNet.PowerManager.PMControl.info/1","doc":"Return information about the specified power manager\n\nNOTE: the map returned may change in the future","ref":"VintageNet.PowerManager.PMControl.html#info/1"},{"type":"function","title":"VintageNet.PowerManager.PMControl.pet_watchdog/1","doc":"Pet watchdog\n\nCall this whenever the network connection is in a good state. If it has\nnot been called by the watchdog timeout, the device will be rebooted.","ref":"VintageNet.PowerManager.PMControl.html#pet_watchdog/1"},{"type":"function","title":"VintageNet.PowerManager.PMControl.power_off/1","doc":"Power off\n\nThis is called when VintageNet stops using an interface. The current state in\nthe power management state machine determines how this is handled. For\nexample, the power could already be off.","ref":"VintageNet.PowerManager.PMControl.html#power_off/1"},{"type":"function","title":"VintageNet.PowerManager.PMControl.power_on/1","doc":"Power on\n\nThis should be called whenever an interface should be powered on. It\ncan be called more than once. If you want the network interface to\nbe on, it is always safe to call this. An internal state machine will\nignore redundant calls.","ref":"VintageNet.PowerManager.PMControl.html#power_on/1"},{"type":"function","title":"VintageNet.PowerManager.PMControl.send_message/2","doc":"Send an arbitrary message to the power manager for an interface\n\nThis will be received by the PowerManager's `handle_info/2` callback.","ref":"VintageNet.PowerManager.PMControl.html#send_message/2"},{"type":"function","title":"VintageNet.PowerManager.PMControl.start_link/1","doc":"Start up a server\n\nThis is intended to be called via `VintageNet.PowerManager.Supervisor`\n\nArguments:\n\n* `:impl` - the module that implements PowerManager\n* `:impl_args` - arguments to pass to the PowerManager's `init/1` call","ref":"VintageNet.PowerManager.PMControl.html#start_link/1"},{"type":"module","title":"VintageNet.PowerManager.Supervisor","doc":"Supervision for all of the power management controllers","ref":"VintageNet.PowerManager.Supervisor.html"},{"type":"function","title":"VintageNet.PowerManager.Supervisor.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"VintageNet.PowerManager.Supervisor.html#child_spec/1"},{"type":"function","title":"VintageNet.PowerManager.Supervisor.start_link/1","doc":"","ref":"VintageNet.PowerManager.Supervisor.html#start_link/1"},{"type":"module","title":"VintageNet.PredictableInterfaceName","doc":"Handles predictable interface names by subscribing to the property table and\nrenaming matching interface names based on the configuration in application\nenvironment.","ref":"VintageNet.PredictableInterfaceName.html"},{"type":"function","title":"VintageNet.PredictableInterfaceName.built_in?/1","doc":"Return whether an ifname is a built-in one\n\nBuilt-in names start with `eth`, `wlan`, etc. and cannot be used\nas interfaces names when using the predictable networking feature.\n\nExamples:\n\n    iex> PredictableInterfaceName.built_in?(\"wlan0\")\n    true\n\n    iex> PredictableInterfaceName.built_in?(\"eth50\")\n    true\n\n    iex> PredictableInterfaceName.built_in?(\"lan\")\n    false","ref":"VintageNet.PredictableInterfaceName.html#built_in?/1"},{"type":"function","title":"VintageNet.PredictableInterfaceName.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"VintageNet.PredictableInterfaceName.html#child_spec/1"},{"type":"function","title":"VintageNet.PredictableInterfaceName.precheck/1","doc":"Called before interface configuration.\n\nFirst checks if vintage_net is configured to use predictable interface names,\nif so checks the given ifname for \"common\" naming schemes.\n\nInstead of a boolean this function returns `:ok` on success, and `{:error,\nnot_predictable_interface_name}` on failure. This is done to allow usage in\n`with` chains.","ref":"VintageNet.PredictableInterfaceName.html#precheck/1"},{"type":"function","title":"VintageNet.PredictableInterfaceName.start_link/1","doc":"","ref":"VintageNet.PredictableInterfaceName.html#start_link/1"},{"type":"type","title":"VintageNet.PredictableInterfaceName.hw_path_config/0","doc":"hw_path to a user supplied ifname mapping","ref":"VintageNet.PredictableInterfaceName.html#t:hw_path_config/0"},{"type":"module","title":"VintageNet.Resolver.ResolvConf","doc":"Utilities for creating resolv.conf file contents","ref":"VintageNet.Resolver.ResolvConf.html"},{"type":"function","title":"VintageNet.Resolver.ResolvConf.to_config/2","doc":"Convert the name server information to resolv.conf contents","ref":"VintageNet.Resolver.ResolvConf.html#to_config/2"},{"type":"function","title":"VintageNet.Resolver.ResolvConf.to_name_server_list/2","doc":"","ref":"VintageNet.Resolver.ResolvConf.html#to_name_server_list/2"},{"type":"type","title":"VintageNet.Resolver.ResolvConf.additional_name_servers/0","doc":"","ref":"VintageNet.Resolver.ResolvConf.html#t:additional_name_servers/0"},{"type":"type","title":"VintageNet.Resolver.ResolvConf.entry/0","doc":"Name resolver settings for an interface","ref":"VintageNet.Resolver.ResolvConf.html#t:entry/0"},{"type":"type","title":"VintageNet.Resolver.ResolvConf.entry_map/0","doc":"All entries","ref":"VintageNet.Resolver.ResolvConf.html#t:entry_map/0"},{"type":"type","title":"VintageNet.Resolver.ResolvConf.name_server_info/0","doc":"","ref":"VintageNet.Resolver.ResolvConf.html#t:name_server_info/0"},{"type":"module","title":"VintageNet.Route","doc":"Types for handling routing tables","ref":"VintageNet.Route.html"},{"type":"type","title":"VintageNet.Route.default_route/0","doc":"A default route entry\n\nThe IP address is the default gateway","ref":"VintageNet.Route.html#t:default_route/0"},{"type":"type","title":"VintageNet.Route.entries/0","doc":"A list of routing table entries","ref":"VintageNet.Route.html#t:entries/0"},{"type":"type","title":"VintageNet.Route.entry/0","doc":"A routing table entry\n\nThis can be turned into real Linux IP routing table entry.","ref":"VintageNet.Route.html#t:entry/0"},{"type":"type","title":"VintageNet.Route.local_route/0","doc":"A local route entry\n\nThis is for routing packets to the LAN","ref":"VintageNet.Route.html#t:local_route/0"},{"type":"type","title":"VintageNet.Route.metric/0","doc":"Metric (priority) for a routing table entry","ref":"VintageNet.Route.html#t:metric/0"},{"type":"type","title":"VintageNet.Route.route_metric_fun/0","doc":"Compute a route metric value from information about the interface\n\nSee `VintageNet.Route.DefaultMetric.compute_metric/2` for an example. This can be\nset using the `:route_metric_fun` application environment key.","ref":"VintageNet.Route.html#t:route_metric_fun/0"},{"type":"type","title":"VintageNet.Route.rule/0","doc":"An IP route rule\n\nIf the source address matches the 3rd element, then use the routing table specified by the\n2nd element.","ref":"VintageNet.Route.html#t:rule/0"},{"type":"type","title":"VintageNet.Route.table_index/0","doc":"Linux routing table index\n\n`:main` is table 254, `:local` is table 255. `:default` is normally the same as `:main`.","ref":"VintageNet.Route.html#t:table_index/0"},{"type":"module","title":"VintageNet.Route.Calculator","doc":"This module computes the desired routing table contents\n\nIt's used by the RouteManager to update the Linux routing tables when interfaces\ncome online or change state. See the RouteManager docs for a discussion of how\nroutes are configured.\n\nThe functions in this module have no side effects so that it's easier\nto test that routing scenarios result in correct Linux routing table\nconfigurations.","ref":"VintageNet.Route.Calculator.html"},{"type":"function","title":"VintageNet.Route.Calculator.compute/3","doc":"Compute a Linux routing table configuration\n\nThe entries are ordered so that List.myers_difference/2 can be used to\nminimize the routing table changes.","ref":"VintageNet.Route.Calculator.html#compute/3"},{"type":"function","title":"VintageNet.Route.Calculator.init/0","doc":"Initialize state carried between calculations","ref":"VintageNet.Route.Calculator.html#init/0"},{"type":"function","title":"VintageNet.Route.Calculator.rule_table_index_range/0","doc":"Return the table indices used for routing based on source IP.","ref":"VintageNet.Route.Calculator.html#rule_table_index_range/0"},{"type":"type","title":"VintageNet.Route.Calculator.interface_infos/0","doc":"","ref":"VintageNet.Route.Calculator.html#t:interface_infos/0"},{"type":"type","title":"VintageNet.Route.Calculator.table_indices/0","doc":"","ref":"VintageNet.Route.Calculator.html#t:table_indices/0"},{"type":"module","title":"VintageNet.Route.DefaultMetric","doc":"Default module for prioritizing network interfaces\n\nThe priority order is:\n\n1. Internet-connected interfaces are chosen before LAN-connected interfaces\n2. Wired Ethernet, then wifi, then mobile and then any other interfaces\n3. The interface's weight is used to resolve ties. By default, the weight is\n   derived from the interfaces index. E.g., `eth0`'s weight is 0 (highest priority)\n   and `eth1`'s weight is 1 (next highest)","ref":"VintageNet.Route.DefaultMetric.html"},{"type":"function","title":"VintageNet.Route.DefaultMetric.compute_metric/2","doc":"Compute the routing metric for an interface with a status\n\nThis uses the prioritization list to figure out what number should\nbe used for the Linux routing table metric. It could also be `:disabled`\nto indicate that a route shouldn't be added to the Linux routing tables\nat all.","ref":"VintageNet.Route.DefaultMetric.html#compute_metric/2"},{"type":"module","title":"VintageNet.Route.IPRoute","doc":"This module knows how to invoke the `ip` command to modify the Linux routing tables","ref":"VintageNet.Route.IPRoute.html"},{"type":"function","title":"VintageNet.Route.IPRoute.add_default_route/4","doc":"Add a default route","ref":"VintageNet.Route.IPRoute.html#add_default_route/4"},{"type":"function","title":"VintageNet.Route.IPRoute.add_local_route/5","doc":"Add a local route","ref":"VintageNet.Route.IPRoute.html#add_local_route/5"},{"type":"function","title":"VintageNet.Route.IPRoute.add_rule/2","doc":"Add a source IP address -> routing table rule","ref":"VintageNet.Route.IPRoute.html#add_rule/2"},{"type":"function","title":"VintageNet.Route.IPRoute.clear_a_local_route/1","doc":"Clear one local route generically","ref":"VintageNet.Route.IPRoute.html#clear_a_local_route/1"},{"type":"function","title":"VintageNet.Route.IPRoute.clear_a_local_route/5","doc":"Clear one local route","ref":"VintageNet.Route.IPRoute.html#clear_a_local_route/5"},{"type":"function","title":"VintageNet.Route.IPRoute.clear_a_route/0","doc":"Clear one default route out of the main table for any interface","ref":"VintageNet.Route.IPRoute.html#clear_a_route/0"},{"type":"function","title":"VintageNet.Route.IPRoute.clear_a_route/2","doc":"Clear one default route that goes to the specified interface","ref":"VintageNet.Route.IPRoute.html#clear_a_route/2"},{"type":"function","title":"VintageNet.Route.IPRoute.clear_a_rule/1","doc":"Clear out one rule","ref":"VintageNet.Route.IPRoute.html#clear_a_rule/1"},{"type":"function","title":"VintageNet.Route.IPRoute.clear_all_routes/0","doc":"Clear all routes on all interfaces","ref":"VintageNet.Route.IPRoute.html#clear_all_routes/0"},{"type":"function","title":"VintageNet.Route.IPRoute.clear_all_rules/1","doc":"Clear all rules that select the specified table or tables","ref":"VintageNet.Route.IPRoute.html#clear_all_rules/1"},{"type":"module","title":"VintageNet.Route.InterfaceInfo","doc":"Routing information for an interface","ref":"VintageNet.Route.InterfaceInfo.html"},{"type":"type","title":"VintageNet.Route.InterfaceInfo.t/0","doc":"Routing information\n\n* `:default_gateway` - default gateway IP address or `nil` if there isn't one\n* `:ip_subnets` - zero or more IP addresses and prefix lengths for what's on this LAN\n* `:weight` - a value to pass on when calculating the routing table metric for weighting\n  an interface over another one\n* `:interface_type` - a rough categorization of the interface between `:ethernet`, `:wifi`,\n  `:cellular`, etc. based on the name. See `VintageNet.Interface.NameUtilities`.\n* `:status` - whether the interface is `:disconnected`, `:lan`, or `:internet`","ref":"VintageNet.Route.InterfaceInfo.html#t:t/0"},{"type":"type","title":"VintageNet.Route.InterfaceInfo.weight/0","doc":"A weight that can be used to differentiate two interfaces that would otherwise be the same priority\n\nLower weights are higher priority.","ref":"VintageNet.Route.InterfaceInfo.html#t:weight/0"},{"type":"module","title":"VintageNet.Route.Properties","doc":"This module contains helpers for updating the global routing properties.\n\nThese include:\n* `[\"available_interfaces\"]`\n* `[\"connection\"]`","ref":"VintageNet.Route.Properties.html"},{"type":"function","title":"VintageNet.Route.Properties.update_available_interfaces/1","doc":"Update the available_interfaces property based on the low level routes\n\nThis function orders interfaces based on metric just like Linux does","ref":"VintageNet.Route.Properties.html#update_available_interfaces/1"},{"type":"function","title":"VintageNet.Route.Properties.update_best_connection/1","doc":"Update the overall connection status\n\n`:disconnected` < `:lan` < `:internet`","ref":"VintageNet.Route.Properties.html#update_best_connection/1"},{"type":"function","title":"VintageNet.Route.Properties.update_connection_status/1","doc":"Update every interface's connection status","ref":"VintageNet.Route.Properties.html#update_connection_status/1"},{"type":"module","title":"VintageNet.RouteManager","doc":"This module manages the default route.\n\nDevices with more than one network interface may have more than one\nway of reaching the Internet. The routing table decides which interface\nan IP packet should use by looking at the \"default route\" entries.\nOne interface is chosen.\n\nSince not all interfaces are equal, we'd like Linux to pick the\nfastest and lowest latency one. for example, one could\nprefer wired Ethernet over WiFi and prefer WiFi over a cellular\nconnection. This module lets you specify an ordering for interfaces\nand sets up the routes based on this ordering.\n\nThis module also handles networking failures. One failure that\nLinux can't figure out on its own is whether an interface can\nreach the Internet. Internet reachability is handled elsewhere\nlike in the `ConnectivityChecker` module. This module should be\ntold reachability status so that it can properly order default\nroutes so that the best reachable interface is used.\n\nIMPORTANT: This module uses priority-based routing. Make sure the\nfollowing kernel options are enabled:\n\n```text\nCONFIG_IP_ADVANCED_ROUTER=y\nCONFIG_IP_MULTIPLE_TABLES=y\n```","ref":"VintageNet.RouteManager.html"},{"type":"function","title":"VintageNet.RouteManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"VintageNet.RouteManager.html#child_spec/1"},{"type":"function","title":"VintageNet.RouteManager.clear_route/1","doc":"Clear out the default gateway for an interface.","ref":"VintageNet.RouteManager.html#clear_route/1"},{"type":"function","title":"VintageNet.RouteManager.refresh_route_metrics/0","doc":"Refresh route metrics for all interfaces.","ref":"VintageNet.RouteManager.html#refresh_route_metrics/0"},{"type":"function","title":"VintageNet.RouteManager.set_connection_status/3","doc":"Set the connection status on an interface.\n\nChanging the connection status can re-prioritize routing. The\nspecified interface doesn't need to have a default route.","ref":"VintageNet.RouteManager.html#set_connection_status/3"},{"type":"function","title":"VintageNet.RouteManager.set_route/3","doc":"Set the default route for an interface.\n\nThis replaces any existing routes on that interface","ref":"VintageNet.RouteManager.html#set_route/3"},{"type":"function","title":"VintageNet.RouteManager.start_link/1","doc":"Start the route manager\n\nOptions:\n\n* `:route_metric_fun` - a 2-arity function that takes a ifname and `VintageNet.Route.InterfaceInfo`\n  and returns `VintageNet.Route.metric()`","ref":"VintageNet.RouteManager.html#start_link/1"},{"type":"function","title":"VintageNet.RouteManager.stop/0","doc":"Stop the route manager.","ref":"VintageNet.RouteManager.html#stop/0"},{"type":"behaviour","title":"VintageNet.Technology","doc":"Technologies define how network interface types work\n\nVintageNet comes with several built-in technologies, but more can be\nadded or existing ones modified by implementing the `Technology` behaviour.","ref":"VintageNet.Technology.html"},{"type":"callback","title":"VintageNet.Technology.check_system/1","doc":"Check that the system has all of the required programs for this technology\n\nThis is intended to help identify missing programs without configuring a\nnetwork.","ref":"VintageNet.Technology.html#c:check_system/1"},{"type":"callback","title":"VintageNet.Technology.ioctl/3","doc":"Handle an ioctl that has been requested on the network interface\n\nThe function runs isolated in its own process and only one ioctl is\nguaranteed to be running at a time. `VintageNet` will handle crashes and\nhangs and unceremoniously kill the ioctl if the user changes their mind and\nreconfigures the network interface.\n\nIoctl support is optional. Examples of `ioctl`s include:\n\n* `:scan` - scan for WiFi networks\n* `:statistics` - return a map of network statistics","ref":"VintageNet.Technology.html#c:ioctl/3"},{"type":"function","title":"VintageNet.Technology.module_from_config!/1","doc":"Helper to fetch the Technology implementation from a configuration","ref":"VintageNet.Technology.html#module_from_config!/1"},{"type":"callback","title":"VintageNet.Technology.normalize/1","doc":"Normalize a configuration\n\nTechnologies use this to update input configurations to a canonical\nrepresentation. This includes things like inserting default fields,\nconverting IP addresses passed in as strings to tuples, and deriving\nparameters so that they need not be derived again in the future.\n\nConfiguration errors raise exceptions.","ref":"VintageNet.Technology.html#c:normalize/1"},{"type":"callback","title":"VintageNet.Technology.to_raw_config/3","doc":"Convert a technology-specific configuration to one for VintageNet\n\nThe `config` is the normalized configuration map (`normalize/1` will have\nbeen called at some point so the technology does not need to call it again).\n\nThe `opts` parameter contains VintageNet's application environment. This\ncontains potentially useful file paths and other information.\n\nConfiguration errors raise exceptions. Errors should be infrequent, though,\nsince VintageNet will call `normalize/1` first and expects most errors to be\ncaught by it.","ref":"VintageNet.Technology.html#c:to_raw_config/3"},{"type":"module","title":"VintageNet.Technology.Null","doc":"An interface with this technology is unconfigured\n\nIf this was due to an error, the reason field will have more information.","ref":"VintageNet.Technology.Null.html"},{"type":"extras","title":"README","doc":"![VintageNet Logo](assets/vintage_net.png)\n\n[![Hex version](https://img.shields.io/hexpm/v/vintage_net.svg \"Hex version\")](https://hex.pm/packages/vintage_net)\n[![API docs](https://img.shields.io/hexpm/v/vintage_net.svg?label=hexdocs \"API docs\")](https://hexdocs.pm/vintage_net/VintageNet.html)\n[![CircleCI](https://circleci.com/gh/nerves-networking/vintage_net.svg?style=svg)](https://circleci.com/gh/nerves-networking/vintage_net)\n\n> **_NOTE:_**  If you've been using `vintage_net` `v0.6.x` or earlier, we split\n> out network technology support out to separate libraries in `v0.7.0`. You'll\n> need to add those libraries to your `mix` dependency list and rename some\n> atoms.  Configurations stored on deployed devices will be automatically\n> updated.  See the [v0.7.0 release\n> notes](https://github.com/nerves-networking/vintage_net/releases/tag/v0.7.0)\n> for details.\n\n`VintageNet` is network configuration library built specifically for [Nerves\nProject](https://nerves-project.org) devices. It has the following features:\n\n* Ethernet and WiFi support included. Extendible to other technologies\n* Default configurations specified in your Application config\n* Runtime updates to configurations are persisted and applied on next boot\n  (configurations are obfuscated by default to hide WiFi passphrases)\n* Simple subscription to network status change events\n* Connect to multiple networks at a time and prioritize which interfaces are\n  used (Ethernet over WiFi over cellular)\n* Internet connection monitoring and failure detection\n* Predictable network interface names\n\n> **TL;DR:** Don't care about any of this and just want the string to copy/paste\n> to set up networking? See the [VintageNet Cookbook](https://hexdocs.pm/vintage_net/cookbook.html).\n\nThe following network configurations are supported:\n\n* [x] Wired Ethernet, IPv4 DHCP\n* [x] Wired Ethernet, IPv4 static IP\n* [x] WiFi password-less and WEP\n* [x] WPA2 PSK and EAP\n* [x] USB gadget mode Ethernet, IPv4 DHCP server to supply host IP address\n* [x] Cellular networks (see\n  [`vintage_net_qmi`](https://hex.pm/packages/vintage_net_qmi) and\n  [`vintage_net_mobile`](https://hex.pm/packages/vintage_net_mobile) for details)\n* [x] WiFi AP mode\n* [ ] IPv6 - Partially supported. SLAAC configuration works.\n\n`vintage_net` takes a different approach to networking from `nerves_network`.\nIts focus is on building and applying network configurations. Where\n`nerves_network` provided configurable state machines, `vintage_net` turns\nhuman-readable configurations into everything from configuration files and calls\nto [`ip`](https://linux.die.net/man/8/ip) to starting up networking `GenServers`\nand routing table updates. This makes it easier to add support for new network\ntechnologies and features. While Elixir and Erlang were great to implement\nnetwork protocols in, it was frequently more practical to reuse embedded Linux\nimplementations. Importantly, though, `vintage_net` monitors Linux daemons under\nits OTP supervision tree so failures on both the \"C\" and Elixir sides propagate\nin the expected ways.\n\nAnother important difference is that `VintageNet` doesn't attempt to make\nincremental modifications to configurations. It completely tears down an\ninterface's connection and then brings up new configurations in a fresh state.\nNetwork reconfiguration is assumed to be an infrequent event so while this can\ncause a hiccup in the network connectivity, it removes state machine code that\nmade `nerves_network` hard to maintain.","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"First, if you're modifying an existing project, you will need to remove\n`nerves_network` and `nerves_init_gadget`. `vintage_net` doesn't work with\neither of them. You'll get an error if any project references those packages.\n\nThere are two routes to integrating `vintage_net`:\n\n1. Use [nerves_pack](https://hex.pm/packages/nerves_pack). `nerves_pack` is like\n   `nerves_init_gadget`, but for `vintage_net`.\n2. Copy and paste from the\n   [Nerves hello WiFi example](https://github.com/nerves-project/nerves_examples/tree/main/hello_wifi)\n\nThe next step is to make sure that your Nerves system is compatible. The\nofficial Nerves systems released after 12/11/2019 work without modification. If\nrolling your own Nerves port, you will need the following Linux kernel options\nenabled:\n\n* `CONFIG_IP_ADVANCED_ROUTER=y`\n* `CONFIG_IP_MULTIPLE_TABLES=y`\n\nThen make sure that you have the following Busybox options enabled:\n\n* `CONFIG_IFCONFIG=y` - `ifconfig` ifconfig\n* `CONFIG_UDHCPC=y` - `udhcpc` DHCP Client\n* `CONFIG_UDHCPD=y` - `udhcpd` DHCP Server (optional)\n\nFinally, you'll need to choose what network connection technologies that you\nwant available in your firmware. If using `nerves_pack`, you'll get support for\nwired Ethernet, WiFi, and USB gadget networking automatically. Otherwise, add\none or more of the following to your dependency list:\n\n* [`vintage_net_ethernet`](https://github.com/nerves-networking/vintage_net_ethernet) - Standard wired Ethernet\n* [`vintage_net_wifi`](https://github.com/nerves-networking/vintage_net_wifi) - Client configurations for 802.11 WiFi\n* [`vintage_net_direct`](https://github.com/nerves-networking/vintage_net_direct) - Direct connections like those used for USB gadget\n* [`vintage_net_qmi`](https://github.com/nerves-networking/vintage_net_qmi) - Support USB-connected cellular modems\n* [`vintage_net_mobile`](https://github.com/nerves-networking/vintage_net_mobile) - Support UART-connected cellular modems","ref":"readme.html#installation"},{"type":"extras","title":"Configuration - README","doc":"`VintageNet` has many application configuration keys. Most defaults are fine. At\na minimum, you'll want to specify a default configuration and default regulatory\ndomain if using WiFi. In your main `config.exs`, add the following:\n\n```elixir\nconfig :vintage_net,\n  regulatory_domain: \"US\",\n  config: [\n    {\"eth0\", %{type: VintageNetEthernet, ipv4: %{method: :dhcp}}},\n    {\"wlan0\", %{type: VintageNetWiFi}}\n  ]\n```\n\nThis sets the regulatory domain to the US (set to your [ISO 3166-1 alpha-2\ncountry code](https://en.wikipedia.org/wiki/ISO_3166-1_alpha-2). This code is\npassed on to the drivers for WiFi and other wireless networking technologies so\nthat they comply with local regulations. If you need a global default, set to\n\"00\" or don't set at all.  Unfortunately, this may mean that an access point\nisn't visible if it is running on a frequency that's allowed in your country,\nbut not globally.\n\nThe `config` section is a list of network configurations. The one shown above\nconfigures DHCP on wired Ethernet and minimally starts up a WiFi LAN so that\nit's possible to scan for networks. The typical setup is to provide generic\ndefaults here. Static IP addresses, WiFi SSIDs and credentials are more\nappropriately configured at run-time. `VintageNet` persists configurations too.\nDetails on network configuration are described later.\n\nThe following table describes the other application config keys.\n\nKey                | Description\n ----------------- | ---------------------------\nconfig             | A list of default network configurations\ntmpdir             | Path to a temporary directory for VintageNet\nudhcpc_handler     | Module for handling notifications from `udhcpc`\nresolvconf         | Path to `/etc/resolv.conf`\npersistence        | Module for persisting network configurations\npersistence_dir    | Path to a directory for storing persisted configurations\npersistence_secret | A 16-byte secret or a function or MFArgs (module, function, arguments tuple) for getting a secret\ninternet_host_list | IP address or hostnames and ports to try to connect to for checking Internet connectivity. Defaults to a list of large public DNS providers. E.g., `[{{1, 1, 1, 1}, 53}]`.\nregulatory_domain  | ISO 3166-1 alpha-2 country (`00` for global, `US`, etc.)\nadditional_name_servers     | List of DNS servers to be used in addition to any supplied by an interface. E.g., `[{1, 1, 1, 1}, {8, 8, 8, 8}]`\nroute_metric_fun   | Customize how network interfaces are prioritized. See `VintageNet.Route.DefaultMetric.compute_metric/2`","ref":"readme.html#configuration"},{"type":"extras","title":"Network interface configuration - README","doc":"`VintageNet` supports several network technologies out of the box and\nthird-party libraries can provide more via the `VintageNet.Technology`\nbehaviour.\n\nConfigurations are Elixir maps. These are specified in three places:\n\n1. The `vintage_net` application config (e.g., your `config.exs`)\n2. Locally saved configuration (see the `VintageNet.Persistence` behaviour for\n   replacing the default)\n3. Calling `VintageNet.configure/2` to change the configuration at run-time\n\nWhen `vintage_net` starts, it applies saved configurations first and if any\nthing is wrong with those configs, it reverts to the application config. A good\npractice is to have safe defaults for all network interfaces in the application\nconfig.\n\nThe only required key in the configuration maps is `:type`. All other keys\nfollow from the type. `:type` should be set to a module that implements the\n`VintageNet.Technology` behaviour. The following are common technologies:\n\n* [`VintageNetEthernet`](https://github.com/nerves-networking/vintage_net_ethernet) - Standard wired Ethernet\n* [`VintageNetWiFi`](https://github.com/nerves-networking/vintage_net_wifi) - Client configurations for 802.11 WiFi\n* [`VintageNetDirect`](https://github.com/nerves-networking/vintage_net_direct) - Direct connections like those used for USB gadget\n  connections\n* `VintageNet.Technology.Null` - An empty configuration useful for turning off a\n  configuration\n\nSee the links above for specific documentation.","ref":"readme.html#network-interface-configuration"},{"type":"extras","title":"Persistence - README","doc":"By default, VintageNet stores network configuration to disk. If you are\nmigrating from `nerves_network` you may already have a persistence\nimplementation. To disable the default persistence, configure `vintage_net` as\nfollows:\n\n```elixir\nconfig :vintage_net,\n  persistence: VintageNet.Persistence.Null\n```","ref":"readme.html#persistence"},{"type":"extras","title":"Debugging - README","doc":"Debugging networking issues is not fun. When you're starting out with\n`vintage_net`, it is highly recommended to connect to your target using a method\nthat doesn't require networking to work. This could be a UART connection to an\nIEx console on a Nerves device or maybe just hooking up a keyboard and monitor.\n\nIf having trouble, first check `VintageNet.info()` to verify the configuration\nand connection status:\n\n```elixir\niex> VintageNet.info\nVintageNet 0.3.0\n\nAll interfaces:       [\"eth0\", \"lo\", \"tap0\", \"wlan0\"]\nAvailable interfaces: [\"eth0\", \"wlan0\"]\n\nInterface eth0\n  Type: VintageNetEthernet\n  Present: true\n  State: :configured\n  Connection: :internet\n  Configuration:\n    %{ipv4: %{method: :dhcp}, type: VintageNetEthernet}\n\nInterface wlan0\n  Type: VintageNetWiFi\n  Present: true\n  State: :configured\n  Connection: :internet\n  Configuration:\n    %{\n      ipv4: %{method: :dhcp},\n      type: VintageNetWiFi,\n      wifi: %{\n        key_mgmt: :wpa_psk,\n        mode: :infrastructure,\n        psk: \"******\",\n        ssid: \"MyLAN\"\n      }\n    }\n```\n\nIf you're using [Toolshed](https://github.com/fhunleth/toolshed), try running\nthe following:\n\n```elixir\niex> ifconfig\nlo: flags=[:up, :loopback, :running]\n    inet 127.0.0.1  netmask 255.0.0.0\n    inet ::1  netmask ffff:ffff:ffff:ffff:ffff:ffff:ffff:ffff\n    hwaddr 00:00:00:00:00:00\n\neth0: flags=[:up, :broadcast, :running, :multicast]\n    inet 192.168.9.131  netmask 255.255.255.0  broadcast 192.168.9.255\n    inet fe80::6264:5ff:fee1:4045  netmask ffff:ffff:ffff:ffff::\n    hwaddr 60:64:05:e1:40:45\n\nwlan0: flags=[:up, :broadcast, :running, :multicast]\n    inet 192.168.9.175  netmask 255.255.255.0  broadcast 192.168.9.255\n    inet fe80::20c:e7ff:fe11:3d46  netmask ffff:ffff:ffff:ffff::\n    hwaddr 00:0c:e7:11:3d:46\n```\n\nOr ping:\n\n```elixir\niex> ping \"nerves-project.com\"\nPress enter to stop\nResponse from nerves-project.com (96.126.123.244): time=48.87ms\nResponse from nerves-project.com (96.126.123.244): time=42.856ms\nResponse from nerves-project.com (96.126.123.244): time=43.097ms\n```\n\nYou can also specify an interface to use with `ping`:\n\n```elixir\niex> ping \"nerves-project.com\", ifname: \"wlan0\"\nPress enter to stop\nResponse from nerves-project.com (96.126.123.244): time=57.817ms\nResponse from nerves-project.com (96.126.123.244): time=46.796ms\n\niex> ping \"nerves-project.com\", ifname: \"eth0\"\nPress enter to stop\nResponse from nerves-project.com (96.126.123.244): time=47.923ms\nResponse from nerves-project.com (96.126.123.244): time=48.688ms\n```\n\nIf it looks like nothing is working, check the logs. On Nerves devices, this\nis frequently done by calling `RingLogger.next` or `RingLogger.attach`.\n\nAt a last resort, please open a GitHub issue. We would be glad to help. We only\nhave one ask and that is that you get us started with an improvement to our\ndocumentation or code so that the next person to run into the issue will have an\neasier time. Thanks!","ref":"readme.html#debugging"},{"type":"extras","title":"Properties - README","doc":"`VintageNet` maintains a key/value store for retrieving information on\nnetworking information:\n\n```elixir\niex> VintageNet.get([\"interface\", \"eth0\", \"connection\"])\n:internet\n\niex> VintageNet.get_by_prefix([])\n[\n  {[\"interface\", \"eth0\", \"connection\"], :internet},\n  {[\"interface\", \"eth0\", \"state\"], :configured},\n  {[\"interface\", \"eth0\", \"type\"], VintageNetEthernet},\n  {[\"interface\", \"wlan0\", \"connection\"], :internet},\n  {[\"interface\", \"wlan0\", \"state\"], :configured},\n  {[\"interface\", \"wlan0\", \"type\"], VintageNetWiFi}\n]\n```\n\nYou can also subscribe to keys and receive a message every time it or one its\nchild keys changes:\n\n```elixir\niex> VintageNet.subscribe([\"interface\", \"eth0\"])\n:ok\n\niex> flush\n{VintageNet, [\"interface\", \"eth0\", \"state\"], :configuring, :configured, %{old_timestamp: 123456, new_timestamp: 124456}}\n```\n\nThe message format is `{VintageNet, name, old_value, new_value, metadata}`\n\nMetadata fields include at least the following:\n\n* `:old_timestamp` - the timestamp for when the property first had the `old_value`\n* `:new_timestamp` - the timestamp when the property changed from the `old_value` to the `new_value`\n\nTimestamps are all captured by calling `System.monotonic_time/0` so they're in `:native` time units.\n\nUse `System.convert_time_unit/3` to convert to the usual time units.\n\n#","ref":"readme.html#properties"},{"type":"extras","title":"Global properties - README","doc":"Property               | Values           | Description\n --------------------- | ---------------- | -----------\n`available_interfaces` | `[eth0, ...]`    | Currently available network interfaces in priority order. E.g., the first one is used by default\n`connection`           | `:disconnected`, `:lan`, `:internet` | The overall network connection status. This is the best status of all interfaces.\n`name_servers`         | `[%{address: ..., from: []}]` | Name server addresses and where VintageNet learned about them\n\n#","ref":"readme.html#global-properties"},{"type":"extras","title":"Common network interface properties - README","doc":"All network interface properties can be found under `[\"interface\", ifname]` in\nthe `PropertyTable`.  The following table lists out properties common to all\ninterfaces:\n\nProperty      | Values              | Description\n ------------ | ------------------- | -----------\n`type`        | `VintageNetEthernet`, etc. | The type of the interface\n`config`      | `%{...}`            | The configuration for this interface\n`state`       | `:configured`, `:configuring`, etc. | The state of the interface from `VintageNet`'s point of view.\n`hw_path`     | `\"/devices/platform/ocp/4a100000.ethernet\"` | This is how Linux internally views the connections going to the interface.\n`connection`  | `:disconnected`, `:lan`, `:internet` | This provides a determination of the Internet connection status\n`lower_up`    | `true` or `false`   | This indicates whether the physical layer is \"up\". E.g., a cable is connected or WiFi associated\n`mac_address` | \"11:22:33:44:55:66\" | The interface's MAC address as a string\n`addresses`   | [address_info]      | This is a list of all of the addresses assigned to this interface\n`dhcp_options` | `%{...}`           | When DHCP is in use, the processed response information and options is stored here. See `t:VintageNet.DHCP.Options.t/0`\n\nSpecific types of interfaces provide more parameters.\n\n#","ref":"readme.html#common-network-interface-properties"},{"type":"extras","title":"Predictable network interface names - README","doc":"When using more than one of the same type of interface, it's possible for Linux\nto reorder their naming. For example, if you have two USB WiFi adapters, one\nwill be named `wlan0` and the other `wlan1`. Which one is first depends on\nthings like when the adapter is found and when kernel modules are loaded.  This\ncan vary between boots and cause a lot of confusion.\n\nThe solution is to rename network interfaces based on characteristics of the\ninterface - such as how it's connected. Then application software refers to the\nnew name rather than names like `wlan0`.  This is a common problem, and\nVintageNet provides support for automatically renaming network interfaces.\n\nIf you're used to `systemd`'s approach to naming interfaces, be aware that\nVintageNet's approach is different: `systemd` has an\n[algorithm](https://www.freedesktop.org/software/systemd/man/systemd.net-naming-scheme.html)\nfor generating names (e.g., `enp4s0`) automatically. VintageNet requires you to\nprovide the names to use (e.q., `internet0`, `lan0`, etc.) and how they map to\nhardware. If VintageNet is confronted with a network interface that is connected\nin a way that it doesn't know about, it will do nothing.\n\n> IMPORTANT: Do not mix and match predictable interface names and\n> non-predictable interface names (`wlan*`, `eth*`, `usb*`, `wwan*`). It is\n> confusing and VintageNet will fight you.\n\nBefore switching to predictable names, find out how your network interfaces are\nconnected. For example, this device has an Ethernet interface and two USB WiFi\ndongles:\n\n```elixir\niex> VintageNet.match([\"interface\", :_, \"hw_path\"])\n[\n  {[\"interface\", \"eth0\", \"hw_path\"], \"/devices/platform/ocp/4a100000.ethernet\"},\n  {[\"interface\", \"lo\", \"hw_path\"], \"/devices/virtual\"},\n  {[\"interface\", \"wlan0\", \"hw_path\"], \"/devices/platform/ocp/47400000.usb/47401c00.usb/musb-hdrc.1/usb2/2-1/2-1:1.0\"},\n  {[\"interface\", \"wlan1\", \"hw_path\"], \"/devices/platform/ocp/47400000.usb/47401400.usb/musb-hdrc.0/usb1/1-1/1-1:1.4\"}\n]\n```\n\nNow update your `config.exs` with the mappings with the `:ifnames` key. Be sure\nto also update the default configuration with the new interface names.\nContinuing the example, imagine that one WiFi adapter supports 802.11 meshing\nand it's guaranteed to be in one USB port on the device. The other USB port can\nhave any of a few types of USB WiFi modules. We need to use predictable naming\nin this case so that meshing is only setup on the adapter that supports it.\n\n```elixir\nconfig :vintage_net,\n  ifnames: [\n    %{\n      hw_path: \"/devices/platform/ocp/4a100000.ethernet\",\n      ifname: \"ethernet0\"\n    },\n    %{\n      hw_path: \"/devices/platform/ocp/47400000.usb/47401c00.usb/musb-hdrc.1/usb2/2-1/2-1:1.0\",\n      ifname: \"primary_wifi\"\n    },\n    %{\n      hw_path: \"/devices/platform/ocp/47400000.usb/47401400.usb/musb-hdrc.0/usb1/1-1/1-1:1.4\",\n      ifname: \"mesh_wifi\"\n    }\n  ],\n  config: [\n    {\"ethernet0\", %{type: VintageNetEthernet}},\n    {\"primary_wifi\", %{type: VintageNetWiFi}},\n    {\"mesh_wifi\", %{type: VintageNetWiFi}}\n  ]\n```\n\n> IMPORTANT: VintageNet has rules about renaming interfaces to prevent confusing\n> errors. Below is a list of reasons it *will not* rename an interface\n\n* `hw_path` matches `/devices/virtual` (such as `lo0`, `ppp0` etc.)\n* A second interface's `hw_path` matches an interface that has already been\n  renamed. This *should* never happen.","ref":"readme.html#predictable-network-interface-names"},{"type":"extras","title":"Internet connectivity checks - README","doc":"VintageNet can check whether a network interface can reach the Internet. This\nhas a few uses:\n\n1. Selecting which network interface is used when a device has more than one. A\n   common example is a device with a backup cellular connection.\n2. Automatically recovering a network interface that has lost connectivity. Some\n   times bouncing the network interface actually works, so doing this\n   automatically can sometimes revive a remote device.\n3. Letting the application know the status of the network connection to provide\n   more helpful information about what's happening.\n\nThe logic for declaring that the Internet is available is:\n\n1. Is there a TCP socket in use on the network interface that has sent and\n   received data from a host that's not on the same subnet? If yes, then the\n   device is Internet-connected.\n2. Get the list of Internet servers to check. See below for the list.\n3. Resolve any domain names in the list. If DNS isn't working, remove them from\n   the list.\n4. Pick a random IP address from the remaining list and \"ping\" it. Technically,\n   VintageNet tries to connect over TCP to a specified port, and if it either\n   connects successfully or gets a port closed response, then the device is\n   Internet-connected.\n5. Wait a bit and then go back to step 1.\n\nThe list of Internet servers to check is critically important. VintageNet uses\nthe `:internet_host_list` key in the application environment for this. The\ndefault setting has many popular name servers in it. The idea being that if you\ncan't reach a name server, the Internet probably isn't going to work well.\n\nIf you are deploying to locations with locked down networks, you'll find that\nthe default setting to test name servers won't work. It is not uncommon to find\na network that blocks popular name servers like 8.8.8.8.\n\nThe recommendation is to set the `:internet_host_list` to include your backend\nserver. If VintageNet can reach it, then presumably your application works and\nhaving VintageNet declare the internet reachable via that network interface is\ncorrect.\n\nFor example,\n\n```elixir\nconfig :vintage_net,\n  internet_host_list: [{\"abcdefghijk-ats.iot.us-east-1.amazonaws.com\", 443}]\n```\n\nThe use of the connectivity checker is specified by the technology. Both the\n`VintageNetEthernet` and `VintageNetWiFi` use the internet connectivity checker.\nThis is selected by adding the `VintageNet.Connectivity.InternetChecker`\nGenServer to the `:child_specs` configuration returned by the technology. E.g.,\n`child_specs: [{VintageNet.Connectivity.InternetChecker, \"eth0\"}]`. Most users\ndo not need to be concerned about this.","ref":"readme.html#internet-connectivity-checks"},{"type":"extras","title":"Power Management - README","doc":"Some devices require additional work to be done for them to become available.\nExamples of this are:\n\n1. Setting a GPIO to enable power to the module\n2. Loading a Linux kernel module that is not automatically loaded via the\n   default mechanisms\n3. Running `usb_modeswitch` to change the USB interface to the appropriate state\n4. Performing an initialization step such as loading firmware\n\nSimilarly, when the network interface is no longer being used, it can be nice to\nundo any steps above.\n\nThis process is referred to as power management in VintageNet even though the\nimplementation may not actually affect power use. To use it, implement the\n`VintageNet.PowerManager` behaviour and register the implementation in your\n`config.exs`.\n\nAdditionally, VintageNet runs a watchdog-like service for network devices that\nsupply `VintageNet.PowerManager` implementations. If the watchdog is not pet\nwithin the timeout period (user-specified and defaults to 60 seconds),\nVintageNet powers the device off and and on. The VintageNet power management\ncode supports mandatory minimum on and off times to prevent damage to hardware\nand also minimize pointless power cycling of hardware.\n\nWhile many network devices are fairly reliable and powering off and on seems\nunnecessary, it can save a trip to the field or a full device reboot.\n\n`VintageNet.info/1` shows the power management state for network interfaces that\nare using this feature.","ref":"readme.html#power-management"},{"type":"extras","title":"VintageNet Cookbook","doc":"# VintageNet Cookbook\n\nNot sure what to pass to `vintage_net`? Take a look below for example\nconfigurations.","ref":"cookbook.html"},{"type":"extras","title":"Compile-time vs. run-time - VintageNet Cookbook","doc":"The examples below all show the options to pass. Where you copy those depends on\nwhether you want the configuration to be a built-in default (i.e., compile-time)\nor whether you want to change it at run-time.\n\nFor compile-time, add something like the following to your `config.exs`:\n\n```elixir\nconfig :vintage_net,\n  config: [\n    {\"eth0\", %{type: VintageNetEthernet, ipv4: %{method: :dhcp}}},\n  ]\n```\n\nBut replace `\"eth0\"` with the interface and the map with the desired\nconfiguration from below.\n\nFor run-time, call\n[`VintageNet.configure`](https://hexdocs.pm/vintage_net/VintageNet.html#configure/3)\nlike this:\n\n```elixir\nVintageNet.configure(\"eth0\", %{type: VintageNetEthernet, ipv4: %{method: :dhcp}})\n```\n\nTo see the current configuration at an IEx prompt, type:\n\n```elixir\niex> VintageNet.info\n```","ref":"cookbook.html#compile-time-vs-run-time"},{"type":"extras","title":"Network interface names - VintageNet Cookbook","doc":"In order to configure a network interface, you will need to know its name.\n`vintage_net` passes names through from Nerves or embedded Linux depending on\nwhere it's being run. The following names are common:\n\n* `\"eth0\"` - The first wired Ethernet interface\n* `\"wlan0\"` - The first WiFi interface\n* `\"usb0\"` - The first virtual Ethernet interface over a USB cable\n\nThe operating system assigns network interface names as it discovers them. If\nyou're running on a device with multiple of the same type of interface, the\ndevice names may be renamed to make them deterministic. An example is `\"enp6s0\"`\nwhere the `p6` and `s0` indicate where the adapter and Ethernet connector\nlocation. Running `ifconfig` on Linux and Nerves can help find these if you are\nunsure.","ref":"cookbook.html#network-interface-names"},{"type":"extras","title":"Wired Ethernet - VintageNet Cookbook","doc":"To use, make sure that you're either using\n[`nerves_pack`](https://hex.pm/packages/nerves_pack) or have\n`:vintage_net_ethernet` in your deps:\n\n```elixir\n  {:vintage_net_ethernet, \"~> 0.8\"}\n```\n\n#","ref":"cookbook.html#wired-ethernet"},{"type":"extras","title":"Wired Ethernet with DHCP - VintageNet Cookbook","doc":"This is regular wired Ethernet - nothing fancy:\n\n```elixir\n%{type: VintageNetEthernet, ipv4: %{method: :dhcp}}\n```\n\n#","ref":"cookbook.html#wired-ethernet-with-dhcp"},{"type":"extras","title":"Wired Ethernet with a static IP - VintageNet Cookbook","doc":"Update the parameters below as appropriate:\n\n```elixir\n%{\n  type: VintageNetEthernet,\n  ipv4: %{\n    method: :static,\n    address: \"192.168.9.232\",\n    prefix_length: 24,\n    gateway: \"192.168.9.1\",\n    name_servers: [\"1.1.1.1\"]\n  }\n}\n```\n\nSee\n[`VintageNet.IP.IPv4Config`](https://hexdocs.pm/vintage_net/VintageNet.IP.IPv4Config.html)\nfor other options. If you're interfacing with other Erlang and Elixir libraries,\nyou may find passing IP tuples more convenient than passing strings. That works\ntoo.","ref":"cookbook.html#wired-ethernet-with-a-static-ip"},{"type":"extras","title":"WiFi - VintageNet Cookbook","doc":"To use, make sure that you're either using\n[`nerves_pack`](https://hex.pm/packages/nerves_pack) or have\n`:vintage_net_wifi` in your deps:\n\n```elixir\n  {:vintage_net_wifi, \"~> 0.8\"}\n```\n\n#","ref":"cookbook.html#wifi"},{"type":"extras","title":"Normal password-protected WiFi (WPA2 PSK) - VintageNet Cookbook","doc":"Most password-protected home networks use WPA2 authentication and pre-shared\nkeys.\n\n```elixir\n%{\n  type: VintageNetWiFi,\n  vintage_net_wifi: %{\n    networks: [\n      %{\n        key_mgmt: :wpa_psk,\n        ssid: \"my_network_ssid\",\n        psk: \"a_passphrase_or_psk\"\n      }\n    ]\n  },\n  ipv4: %{method: :dhcp},\n}\n```\n\nHere are example parameters for an static IP address.\n\n```elixir\n%{\n  type: VintageNetWiFi,\n  vintage_net_wifi: %{\n    networks: [\n      %{\n        key_mgmt: :wpa_psk,\n        ssid: \"my_network_ssid\",\n        psk: \"a_passphrase_or_psk\"\n      }\n    ]\n  },\n  ipv4: %{\n    method: :static,\n    address: \"192.168.9.232\",\n    prefix_length: 24,\n    gateway: \"192.168.9.1\",\n    name_servers: [\"1.1.1.1\"]\n  }\n}\n```\n\nIf you're regularly switching between multiple networks, you can list them all\nunder the `:networks` key. Note that it's currently not possible to mix networks\nthat require static IP addresses with those that use DHCP.\n\n```elixir\n%{\n  type: VintageNetWiFi,\n  vintage_net_wifi: %{\n    networks: [\n      %{\n        key_mgmt: :wpa_psk,\n        ssid: \"my_network_ssid\",\n        psk: \"a_passphrase_or_psk\"\n      },\n      %{\n        key_mgmt: :wpa_psk,\n        ssid: \"another_ssid\",\n        psk: \"a_passphrase_or_psk\"\n      },\n      ...\n    ]\n  },\n  ipv4: %{method: :dhcp},\n}\n```\n\n#","ref":"cookbook.html#normal-password-protected-wifi-wpa2-psk"},{"type":"extras","title":"Enterprise WiFi (PEAPv0/EAP-MSCHAPV2) - VintageNet Cookbook","doc":"Protected EAP (PEAP) is a common authentication protocol for enterprise WiFi networks.\n\n```elixir\n%{\n  type: VintageNetWiFi,\n  vintage_net_wifi: %{\n    networks: [\n      %{\n        key_mgmt: :wpa_eap,\n        ssid: \"my_network_ssid\",\n        identity: \"username\",\n        password: \"password\",\n        eap: \"PEAP\",\n        phase2: \"auth=MSCHAPV2\"\n      }\n    ]\n  },\n  ipv4: %{method: :dhcp}\n}\n```\n\n#","ref":"cookbook.html#enterprise-wifi-peapv0-eap-mschapv2"},{"type":"extras","title":"Enterprise WiFi (EAP-TLS) - VintageNet Cookbook","doc":"TBD\n\n#","ref":"cookbook.html#enterprise-wifi-eap-tls"},{"type":"extras","title":"Hidden WiFi networks - VintageNet Cookbook","doc":"If the access point has been configured to not advertise a network, VintageNetWiFi won't find it. It has to explicitly be told to search for\nit. Add `scan_ssid: 1` to the configuration to do this. For example,\n\n```elixir\n%{\n  type: VintageNetWiFi,\n  vintage_net_wifi: %{\n    networks: [\n      %{\n        key_mgmt: :wpa_psk,\n        ssid: \"my_network_ssid\",\n        psk: \"a_passphrase_or_psk\",\n        scan_ssid: 1\n      }\n    ]\n  },\n  ipv4: %{method: :dhcp},\n}\n```\n\n#","ref":"cookbook.html#hidden-wifi-networks"},{"type":"extras","title":"Access point WiFi - VintageNet Cookbook","doc":"Some WiFi modules can be run in access point mode. This makes it possible to\ncreate configuration wizards and captive portals. Configuration of this is more\ninvolved. Here is a basic configuration:\n\n```elixir\n%{\n  type: VintageNetWiFi,\n  vintage_net_wifi: %{\n    networks: [\n      %{\n        mode: :ap,\n        ssid: \"test ssid\",\n        key_mgmt: :none\n      }\n    ]\n  },\n  ipv4: %{\n    method: :static,\n    address: \"192.168.24.1\",\n    netmask: \"255.255.255.0\"\n  },\n  dhcpd: %{\n    start: \"192.168.24.2\",\n    end: \"192.168.24.10\",\n    options: %{\n      dns: [\"1.1.1.1\", \"1.0.0.1\"],\n      subnet: \"255.255.255.0\",\n      router: [\"192.168.24.1\"]\n    }\n  }\n}\n```\n\nIf you want to use WPA2 on your access point, make the networks map look like\nthis:\n\n```elixir\n  %{\n    mode: :ap,\n    key_mgmt: :wpa_psk,\n    proto: \"RSN\",\n    pairwise: \"CCMP\",\n    group: \"CCMP\",\n    ssid: \"test ssid\",\n    psk: \"secret123\"\n  }\n```\n\nThe `proto: \"RSN\"` entry is important since the `wpa_supplicant` default is\n`WPA` and not `WPA2`.\n\nSee the\n[vintage_net_wizard](https://github.com/nerves-networking/vintage_net_wizard)\nfor an example of a project that uses AP mode and a web server for WiFi\nconfiguration.\n\n#","ref":"cookbook.html#access-point-wifi"},{"type":"extras","title":"Advanced Use of WPA Supplicant - VintageNet Cookbook","doc":"VintageNetWifi supports an \"escape hatch\" of sorts if you need precise control over the contents of the supplicant configuration.\nThe contents of the `wpa_supplicant_conf` will be coppied without validation to the wpa_supplicant.conf file that\nVintageNet manages. Example:\n\n```elixir\n%{\n  type: VintageNetWiFi,\n  vintage_net_wifi: %{\n    wpa_supplicant_conf: \"\"\"\n    network={\n      ssid=\"home\"\n      key_mgmt=WPA-PSK\n      psk=\"very secret passphrase\"\n    }\n    \"\"\"\n  },\n  ipv4: %{method: :dhcp}\n}\n```\n\n#","ref":"cookbook.html#advanced-use-of-wpa-supplicant"},{"type":"extras","title":"Bridged Mesh WiFi - VintageNet Cookbook","doc":"In addition to infrastructure and AP modes, some WiFi modules can form a mesh.\nVintageNet supports the configuration of [802.11s](https://en.wikipedia.org/wiki/IEEE_802.11s) meshes.\nWhile this is the standardize way of forming WiFi meshes, it is not the same as that implemented\nby many access points that advertise WiFi meshing. It also uses the 802.11s routing protocol HWMP. (This is\nnot B.A.T.M.A.N.).\n\nThis section describes two configuration: the first is for the mesh gate and the second is for the mesh\ndevices. The mesh gate bridges the mesh network to the network that connects to the Internet. Mesh\nnodes behave similar to normal clients: after connecting to the network, they request an IP address using\nDHCP. The DHCP request gets routed through the mesh gate and to the DHCP server on the non-mesh\nLAN. It's possible to have multiple mesh gates. Routing through the mesh and the mesh gate is\ntransparent.\n\nThe following configuration is for a mesh gate with one WiFi interface used for the mesh network and a wired network interface, `eth0`, that connects it to the LAN:\n\n```elixir\nmesh0_config = %{\n  type: VintageNetWiFi,\n  vintage_net_wifi: %{\n    user_mpm: 1,\n    # mesh creates a \"virtual\" interface based on\n    # this interface name\n    root_interface: \"wlan0\",\n    networks: [\n      %{\n        key_mgmt: :none,\n        ssid: \"my-mesh\",\n        frequency: 2432,\n        mode: :mesh\n      }\n    ]\n  },\n  # we don't need an ip address on the mesh interface\n  ipv4: %{method: :disabled},\n}\n\n# Bridge configured to bridge eth0 and mesh0 together\nbr0_config = %{\n  type: VintageNetBridge,\n  ipv4: %{method: :dhcp},\n  vintage_net_bridge: %{\n    interfaces: [\"eth0\", \"mesh0\"]\n  }\n}\n\neth0_config = %{\n  type: VintageNetEthernet,\n  # the bridge handles ip addressing\n  ipv4: %{method: :disabled},\n}\n\nVintageNet.configure(\"mesh0\", mesh0_config)\nVintageNet.configure(\"br0\", br0_config)\nVintageNet.configure(\"eth0\", eth0_config)\n```\n\nThis configuration is for devices on the mesh:\n\n```elixir\nmesh0_config = %{\n  type: VintageNetWiFi,\n  vintage_net_wifi: %{\n    user_mpm: 1,\n    # mesh creates a \"virtual\" interface based on\n    # this interface name\n    root_interface: \"wlan0\",\n    networks: [\n      %{\n        key_mgmt: :none,\n        ssid: \"my-mesh\",\n        frequency: 2432,\n        mode: :mesh\n      }\n    ]\n  },\n  # the mesh is bridged on the other\n  # device, so we can use dhcp now\n  ipv4: %{method: :dhcp},\n}\nVintageNet.configure(\"mesh0\", mesh0_config)\n```","ref":"cookbook.html#bridged-mesh-wifi"},{"type":"extras","title":"Network interaction - VintageNet Cookbook","doc":"#","ref":"cookbook.html#network-interaction"},{"type":"extras","title":"Share WAN with other networks - VintageNet Cookbook","doc":"For sharing your WAN connection (e.g. internet access) with other networks\n`iptables` must be installed. Currently this means building a [custom nerves\nsystem](https://hexdocs.pm/nerves/customizing-systems.html). Once this is done\nthe following commands need to be called on each boot:\n\n```elixir\nwan = \"eth0\"\ncmd \"sysctl -w net.ipv4.ip_forward=1\"\ncmd \"iptables -t nat -A POSTROUTING -o #{wan} -j MASQUERADE\"\ncmd \"iptables --append FORWARD --in-interface wlan0 -j ACCEPT\"\n# Only needed if the connection is blocked otherwise (like a default policy of DROP)\ncmd \"iptables -A INPUT -i #{wan} -m state --state RELATED,ESTABLISHED -j ACCEPT\"\n```","ref":"cookbook.html#share-wan-with-other-networks"},{"type":"extras","title":"Common tasks - VintageNet Cookbook","doc":"#","ref":"cookbook.html#common-tasks"},{"type":"extras","title":"Temporarily disable WiFi - VintageNet Cookbook","doc":"`VintageNet` persists configurations by default. Sometimes you just want to\ndisable a network temporarily and then if the device reboots, it reboots to the\nold configuration. The `:persist` option let's you do this:\n\n```elixir\nVintageNet.deconfigure(\"wlan0\", persist: false)\n```\n\nTo get the old configuration back, you have to call `VintageNet.configure/3`\nwith it again (or restart `VintageNet` or reboot).\n\n#","ref":"cookbook.html#temporarily-disable-wifi"},{"type":"extras","title":"Perform some initialization to turn on a network interface - VintageNet Cookbook","doc":"`VintageNet` waits for network interfaces to appear before doing any work.  If\nyou need to perform some work to make the network interface show up, that has to\nbe done elsewhere. If you let `VintageNet` know about this work and allow it to\nturn the network interface off too, it can \"cycle power\" to the interface to get\nit back to a clean state when needed. Here's how:\n\n```elixir\ndefmodule MyPowerManager do\n  @behaviour VintageNet.PowerManager\n\n  @reset_n_gpio 4\n  @power_on_hold_time 5 * 60000\n  @min_powered_off_time 5000\n\n  defstruct reset_n: nil\n\n  @impl VintageNet.PowerManager\n  def init(_args) do\n    {:ok, reset_n} = Circuits.GPIO.open(@reset_n_gpio, :output)\n    {:ok, %__MODULE__{reset_n: reset_n}}\n  end\n\n  @impl VintageNet.PowerManager\n  def power_on(state) do\n    # Do whatever is necessary to turn the network interface on\n    Circuits.GPIO.write(state.reset_n, 1)\n    {:ok, state, @power_on_hold_time}\n  end\n\n  @impl VintageNet.PowerManager\n  def start_powering_off(state) do\n    # If there's a graceful power off, start it here and return\n    # the max time it takes.\n    {:ok, state, 0}\n  end\n\n  @impl VintageNet.PowerManager\n  def power_off(state) do\n    # Disable the network interface\n    Circuits.GPIO.write(state.reset_n, 0)\n    {:ok, state, @min_powered_off_time}\n  end\n```\n\nThen add the following to your `config.exs`:\n\n```elixir\nconfig :vintage_net, power_managers: [{MyPowerManager, ifname: \"wlan0\"}]\n```\n\nVintageNet determines whether devices are ok by use of a watchdog. VintageNet\nand its technology implementations pet the watchdog by calling\n`VintageNet.PowerManager.PMControl.pet_watchdog/1`. This may be insufficient for\nyour application. Options include calling that function in your code regularly\nor modifying the `:watchdog_timeout` in the power manager spec in your\n`config.exs`.\n\nSee `VintageNet.PowerManager` for details.","ref":"cookbook.html#perform-some-initialization-to-turn-on-a-network-interface"},{"type":"extras","title":"Changelog","doc":"# Changelog\n\nThis project follows [Semantic Versioning](https://semver.org/spec/v2.0.0.html).\n\n## [v0.13.5] - 2023-09-13\n\n* Changed\n  * Warn when setting default routes with IP addresses outside of the subnet\n    rather than crashing. Thanks to Ben Murphy for this fix.\n  * Bulletproof force clearing of IP addresses to avoid crashing unnecessarily\n    during cleanup. This was a very rare case.\n\n## [v0.13.4] - 2023-07-07\n\n* Changed\n  * VintageNet configurations are normalized on load now rather than on use.\n    For various reasons, it's useful to support multiple ways of specifying\n    network configurations, but internally VintageNet always converts to one way\n    to simplify use. If you call `VintageNet.get_configuration/1` or\n    `VintageNet.get/2` to look at the config, you'd see the original form and not\n    the normalized one. Now you get the normalized one.\n  * Add `:reason` to the `VintageNet.Technology.Null`. VintageNet uses `Null` to\n    make network interfaces stay unconfigured when requested or when there's an\n    error. The `:reason` key helps you know why rather than forcing you to dig\n    through logs.\n\n* Fixes\n  * When reseting a network configuration to defaults, a settings file was\n    written and immediately erased. That doesn't happen any more.\n\n## [v0.13.3] - 2023-06-10\n\n* Changed\n  * Sort dhcpd options so their order doesn't change in configuration files made\n    by OTP 26. This fixes regression test failures that expected the files to by\n    the same. It shouldn't matter for real use, but it's nice that the files are\n    deterministic just in case.\n  * Ignore unexpected messages to the InterfacesMonitor. This fixes an\n    unnecessary crash/restart that was seen. Errors are still logged.\n\n## [v0.13.2] - 2023-05-15\n\n* Changed\n  * Always set IPv4 broadcast address for static IPv4 configurations. This fixes\n    an issue where the default for subnet broadcast address was not the expected\n    host all-ones address per RFC 922.\n  * Prune out LAN addresses when trying to detect Internet connectivity. This\n    fixes one way that the Internet checker could be tricked by a captive portal\n    that resolves all DNS queries to its portal address.\n  * Fix confusion with `:dhcpd` `:subnet` option. This maps to the subnet mask\n    field when responding to DHCP requests. The word \"subnet\" was interpreted as\n    a subnet which was incorrect for this. `:netmask` is now an alias and\n    examples are fixed.\n\n## [v0.13.1] - 2023-03-15\n\nThis release fixes deprecation warnings when using Elixir 1.15.\n\n## [v0.13.0] - 2023-01-22\n\nThis release has a breaking change if you're using the `UdhcpcHandler`\nbehaviour. This should be a rare use case. Use the `\"dhcp_options\"` property\nnow.\n\n* Changed\n  * Add `\"dhcp_options\"` to interface properties. This lets applications use\n    information provided by DHCP servers in an easy way. It removes the need to\n    process events from `udhcpc` directly, and therefore, the `UdhcpcHandler`\n    behaviour is now a private API and may be removed in the future.\n  * Fix some references to `iodata` that should have been `chardata`.\n\n## [v0.12.2] - 2022-08-04\n\n* Changed\n  * Add `VintageNet.info_as_ansidata/1`. This lets you get the same results as\n    `VintageNet.info`, but in a way that's easy to put on a web page or send\n    to a server, etc.\n  * Support updating network configuration even when it can't be persisted.\n    Previously if there was an error saving the configuration and `persist:\n    true` (the default), the configuration wouldn't be applied. This turned\n    out to be problematic when trying to get some devices fixed. Now the\n    device won't have the right config on reboot, but it can be reached over\n    the network to be fixed.\n\n## [v0.12.1] - 2022-06-01\n\n* Changed\n  * The list of name servers that VintageNet uses when configuring the name\n    resolve is now available by running `VintageNet.get([\"name_servers\"])`.\n\n## [v0.12.0] - 2022-04-27\n\nThis release has two potentially breaking changes:\n\n1. Elixir 1.11 is now the minimum supported Elixir version.\n2. `VintageNet.PropertyTable` has been extracted to its own library and is now\n   just `PropertyTable`. Most users did not use `VintageNet.PropertyTable`\n   directly, but if you did, you'll need to update the references.\n\n* Changed\n  * Extract `VintageNet.PropertyTable` to its own library. Note that many\n    improvements were made to PropertyTable including renaming functions for\n    consistency and changing the events. Code was added to VintageNet to hide\n    these changes for now. Longer term, we'll be making things more consistent,\n    but the hope is that the PropertyTable changes are transparent to VintageNet\n    users in this release.\n  * Support specifying absolute paths to network configuration commands. While\n    this is not preferred, it's useful in some scenarios.\n  * Redact more kinds of secrets in `VintageNet.info`\n\n## [v0.11.5] - 2022-02-18\n\n* Changed\n  * Fix a no function clause exception in the InternetChecker that could happen\n    if no IP addresses were assigned to an interface.\n\n## [v0.11.4] - 2021-12-20\n\n* Changed\n  * Internet connectivity checks can now take domain names. Previously only IP\n    addresses were supported. This change lets you add your own servers to the\n    list since those servers may be more reliable indicators of Internet access\n    in highly firewalled locations. A section on this was added to the README.md\n    with an example.\n\n## [v0.11.3] - 2021-11-18\n\n* Changed\n  * Don't downgrade the connection status on DHCP renewals. Previously, if there\n    was a DHCP renewal, the connection status could go from \"Internet-connected\"\n    \"LAN-connected\". The logic was that IP address and router changes may make\n    the Internet unreachible. The new logic is to assume that the device is\n    still Internet-connected and let the connectivity checker downgrade the\n    status should it be necessary. This not only removes a status hiccup, but\n    also fixes a race between the connectivity checker upgrading the connection\n    and the DHCP notification degrading it.\n  * Improve the `VintageNet.info` error when the `:vintage_net` application\n    stops.\n\n## [v0.11.2] - 2021-10-25\n\n* Added\n  * Added `VintageNet.RouteManager.refresh_route_metrics/0` to recompute the\n    routing table metrics. This is useful if you're supplying your own\n    `:route_metric_fun` and something has changed to make it return a different\n    prioritization. Thanks to @LostKobrakai for this feature.\n\n## [v0.11.1] - 2021-10-01\n\n* Changed\n  * The DNS server ordering is more deterministic now. Global DNS servers are\n    guaranteed to be listed first and in the order specified. VintageNet will\n    also try to preserve the ordering of DNS servers learned through DHCP. This\n    isn't always possible, though. This fixes a hard-to-find error where an\n    a difference in DNS server orderings between to device locations led to\n    different behavior.\n  * Use `VintageNet.ConnectivityChecker.*` in VintageNet. The connectivity\n    checker module change was half made in `v0.11.0`, but the old module name\n    was kept in a couple places to avoid breaking unit tests in other VintageNet\n    libraries. Now it's completely converted. This doesn't affect runtime. Code\n    that references the previous names will still get deprecation warnings like\n    in `v0.11.0`.\n\n## [v0.11.0] - 2021-08-19\n\nThis release should be a safe update for most users. Many routing table and\ninternet connectivity check modules were updated, but the changes were primarily\nin private APIs.\n\n* Added\n  * Support for detecting Internet connectivity on an interface passively by\n    watching tx and rx stats on TCP sockets. For example, if you have a\n    long-lived TCP connection (like for MQTT), the keepalive messages will\n    bump tx and rx counters that will let VintageNet skip testing the connection\n    for connectivity. This reduces traffic on metered connections.\n  * Support for completely overriding route metric calculation. You can now\n    specify a `:route_metric_fun` instead of using the `DefaultMetric`\n    calculator for determining which network interface preferences.\n  * VintageNet property change events now come with timestamps. These are useful\n    for computing state durations and other time-based stats for events.\n\n* Removed\n  * Support for setting route prioritization order. This feature was more\n    limiting that it originally looked. The new `:route_metric_fun` is more\n    straightforward since it lets you explicitly specify orderings and lets\n    decisions be made based on more input data.\n\n* Changed\n  * `VintageNet.Interface.InternetConnectivityChecker` is now\n    `VintageNet.Connectivity.InternetChecker`. Please update any references. Old\n    references will continue to work, but give a deprecation message at runtime.\n\n## [v0.10.5] - 2021-07-12\n\nThis release only contains build system and hex package update. It doesn't\nchange any code and is a safe update.\n\n## [v0.10.4] - 2021-07-06\n\n* Fixed\n  * DHCP renewals would bounce connection status from :internet to :lan and back\n    even when the IP address, subnet, and default gateway didn't change. This\n    could cause a network connectivity hiccup that would happen every 24 hours\n    (a common DHCP lease time). A fix was added to assume internet connectivity\n    was maintained if the DHCP renewal didn't change IP parameters.\n\n## [v0.10.3] - 2021-06-22\n\n* Fixed\n  * Fix regression with tracking udhcpd lease notifications. Leases\n    notifications were being ignored, so if you were monitoring leases to see\n    who was connected, then you wouldn't see any connections without this fix.\n    Thanks to Jon Thacker for reporting this issue.\n  * Fix crashes when the application config is invalid. While the configurations\n    were incorrect and needed to be fixed, it was harder to debug than it should\n    have been. This release logs messages on invalid configs and carries on\n    bringing up left that's valid. Thanks to Matt Ludwigs for this fix.\n\n## [v0.10.2] - 2021-05-20\n\nThis release officially removes support for Elixir 1.7 and Elixir 1.8. It turns\nout that those versions wouldn't have worked in v0.10.0 due to a dependency that\nwas added.\n\n* Added\n  * Add `VintageNet.reset_to_defaults/1` so that it's easy to reset a network\n    interface's configuration to what it would be if `VintageNet.configure` had\n    never been called. The previous \"easy\" way of doing this was to erase the\n    persisted configuration file and reboot.\n  * Clean up interface reachability handling (disconnected vs. lan-connected vs\n    internet-connected). There was an issue where the status was out of sync due\n    a bug in a technology implementation. This is harder to do now. IMPORTANT:\n    if you have a custom technology implementation, calling\n    `VintageNet.RouteManager.set_connection_status` is sufficient. You no\n    longer need to update the status property for your interface. This is not a\n    common need.\n\n## [v0.10.1] - 2021-05-06\n\n* Added\n  * There's now an `:additional_name_servers` global configuration key so that\n    it's possible to force name servers to always be in the list to use. For\n    example, if you don't trust that you'll always get good name servers from\n    DHCP, you can add a few public name servers to this list.\n  * `/etc/resolv.conf` now has nice comments on where configuration items come\n    from. Thanks to Connor Rigby for this idea and implementation.\n\n## [v0.10.0] - 2021-04-06\n\nThis release is mostly backwards compatible. If you have created your own\nVintageNet technology, you may need to update your unit tests. If you are an end\nuser of VintageNet, your code should continue to work unmodified.\n\n* Added\n  * The Internet connectivity check logic now supports a list of IP addresses\n    instead of just one. The default has been updated to include major public\n    DNS providers. The code checks them in succession until one responds. See\n    `:internet_host_list` config key in the README.md if you need to change it.\n  * Only start `udhcpc`/`udhcpd` when the network interface is up. This removes\n    pointless attempts to get an IP address and their associated logs. It\n    reduces connection time for wired Ethernet but doesn't affect WiFI.\n\n* Fixed\n  * Replace Crypto API calls that are no longer included with OTP 24.\n  * Redact SAE passwords\n\n## [v0.9.3] - 2021-02-03\n\n* Fixed\n  * Be more robust to `PowerManager.init/1` failures. While this function\n    shouldn't raise, the effect of it raising was particularly destructive to\n    VintageNet and took down networking.\n  * Update `gen_state_machine` dependency to let the 3.0.0 release be used.\n\n## [v0.9.2] - 2020-10-10\n\n* Fixed\n  * Handle missing commands as errors rather than raising. This makes it\n    a little easier test `vintage_net` and libraries that use it.\n  * Fixes `@doc` tag warnings during compile time\n\n## [v0.9.1] - 2020-07-29\n\n* Fixed\n  * This fixes an issue where system networking binaries were not being resolved\n    according to `vintage_net`'s view of the `PATH`. `vintage_net` looks in the\n    standard directories by default, but it's possible to restrict or add\n    locations.\n\n## [v0.9.0] - 2021-07-24\n\nThis release contains improvements that will not affect you unless you are\nusing a custom `VintageNet.Technology` implementation.\n\n* Added\n  * Add power management support. This adds support for powering on and off\n    network devices and also enables `VintageNet` to restart devices that are\n    not working (if allowed). See `VintageNet.PowerManager` for details.\n\n* Changed\n  * Paths to networking programs like `wpa_supplicant` are no longer passed as\n    opts during configuration. I.e., `:bin_wpa_supplicant`, `:bin_ip`, etc. This\n    was not a generally useful feature since it wasn't possible to include all\n    possible programs. A future plan is to add support for verifying that\n    networking programs exist before trying to configure an interface. Programs\n    should be passed as strings now.\n  * Support for the `:busybox` hex package has been removed. This was useful\n    when networking programs were unavailable on a system, but all official\n    Nerves systems have included them for the past year and `:busybox` required\n    maintenance to keep working and up-to-date.\n\n## [v0.8.0] - 2020-05-29\n\n* Added\n  * [Breaking change for technology implementors] Decouple the network interface\n    name from the one a network technology uses. For example, cellular modems\n    can now have `vintage_net` wait for `wwan0` to appear before setting up a\n    PPP interface (like `ppp0`). All network technology implementations need to\n    be updated to provide `RawConfigs` that list the network interfaces they\n    need to start. This is hard to miss since you'll get a compile error if it\n    affects you.\n  * Deterministic interface naming support - If you have a device with multiple\n    network interfaces of the same type (e.g., multiple WiFi adapters) it is\n    possible for them to switch between being assigned `wlan0` and `wlan1`\n    under some conditions. This feature allows you to map their hardware\n    location to a name of your choosing. See the `README.md` for details.\n  * Add the \"hw_path\" property - For example, `{[\"interface\", \"eth0\", \"hw_path\"], \"/devices/platform/ocp/4a100000.ethernet\"}`\n\n* Fixed\n  * Stop network interface management `GenServers` before running the \"down\"\n    commands. This is most noticeable in reduced log noise on network hiccups\n    and device removals.\n\n## [v0.7.9] - 2020-04-07\n\n* Fixed\n  * Fix IP address being reported for PPP connections. Previously, it was the\n    remote end of the PPP connection rather than the local end.\n  * Fix missing IPv6 address reports. Depending on when IPv6 addresses were set\n    on network interfaces, they might not have been reported. Note that IPv6\n    isn't officially supported by VintageNet yet.\n\n## [v0.7.8] - 2020-04-03\n\n* Added\n  * Store an interface's configuration in the `[\"interface\", ifname, \"config\"]`\n    property. This makes it possible to subscribe to configuration changes (like\n    any other property).\n  * Print out IP addresses with `VintageNet.info/0`\n\n* Fixed\n  * Fixed `VintageNet.get_configuration/1` to return the configuration that will\n    be applied even if it's not the configuration that's currently applied.\n    The previous semantics would break code that made decisions based on the\n    current configurations.\n\n## [v0.7.7] - 2020-03-23\n\n* Added\n  * Added time-in-state to `VintageNet.info`. This lets you see if a connection\n    has bounced at a glance without digging through the logs.\n\n## [v0.7.6] - 2020-03-18\n\n* Fixed\n  * Ensure that `Technology.normalize/1` is always called. Previously, this\n    wasn't guaranteed, and it could result in a surprise when an unnormalized\n    configuration got saved.\n  * Remove duplicate resolv.conf entries on multi-homed devices\n  * Fix warnings found by Elixir 1.10\n\n## [v0.7.5] - 2020-02-10\n\n* Fixed\n  * Fix routing table error when configuring multiple interfaces of the same\n    type.\n  * Fix `VintageNet.info` for when it's called before `vintage_net` is loaded.\n\n## [v0.7.4] - 2020-01-22\n\n* Fixed\n  * Fix `VintageNet.info` crash when displaying AP mode configurations\n  * Save configurations using the `:sync` flag to reduce the chance that they're\n    lost on ungraceful power offs. I.e., people pulling the power cable after\n    device configuration.\n\n## [v0.7.3] - 2020-01-21\n\n* Added\n  * Scrub `VintageNet.info/0` output to avoid accidental disclosure of WiFi\n    credentials\n  * Support options to `deconfigure/2` to mirror those on `configure/2`\n  * Prefix `udhcpc` logs with interface to more easily blame problematic\n    networks\n  * Support IPv4 /32 subnets\n  * Various documentation fixes and improvements\n\n## [v0.7.2] - 2019-12-20\n\n* Bug fix\n  * Remove noisy log message introduced in v0.7.1\n\n## [v0.7.1] - 2019-12-20\n\nThis release fixes an issue where the Internet-connectivity checking code could\ncrash. It was automatically restarted, but that had a side effect of delaying a\nreport that the device was connected AND breaking `mdns_lite`. Both the crash\nand the restart issue were fixed. The `mdns_lite` side effect was due to its\nmulticast group membership being lost so this would affect other multicast-using\ncode.\n\n* Fixed\n  * Fix `:timeout_value` crash in the `InternetConnectivityChecker`\n  * Force clear IPv4 addresses when the DHCP notifies a deconfig event. This\n    occurs on a restart and is quickly followed by a renew. However, if\n    applications don't see this, bounce and don't register their multicast\n    listeners on affected IPv4 address again, they'll lose the subscription.\n\n* Added\n  * Added check for `nerves_network` and `nerves_init_gadget`. If your project\n    pulls these in, it will get a moderately friendly notice to remove them.\n\n## [v0.7.0] - 2019-12-09\n\nThis release moves network technology implementations (WiFi, wired Ethernet,\netc.) into their own projects. This means that they can evolve at their own\npace. It also means that we're finally ready to support the\n`VintageNet.Technology` behaviour as part of the public API so that VintageNet\nusers can add support for network technologies that we haven't gotten to yet.\n\nIMPORTANT: This change is not backwards compatible. You will need to update\nexisting projects to bring in a new dependency. The runtime is backwards\ncompatible. I.e., If you have a networking configuration saved in VintageNet, it\nwill be updated on load. It won't be re-saved, so if you need to revert an\nupdate, it will still work. The next save, though, will use the new naming.\n\nIf you're using `VintageNet.Technology.Gadget`, do the following:\n\n1. Add `{:vintage_net_direct, \"~> 0.7.0\"}` to your `mix.exs` dependencies.\n   You'll notice that references to \"gadget\" have been replaced with the word\n   \"direct\". We think the new naming is more accurate.\n2. Replace all references to `VintageNet.Technology.Gadget` in your code to\n   `VintageNetDirect`. Be aware of aliases and configuration.\n3. If you passed options when configuring the network, the `:gadget` key is\n   now `:vintage_net_direct`. Most users don't pass options.\n\nIf you're using `VintageNet.Technology.Ethernet`, do the following:\n\n1. Add `{:vintage_net_ethernet, \"~> 0.7.0\"}` to your `mix.exs` dependencies.\n2. Replace all references to `VintageNet.Technology.Ethernet` in your code to\n   `VintageNetEthernet`. Be aware of aliases and configuration.\n\nIf you're using `VintageNet.Technology.WiFi`, do the following:\n\n1. Add `{:vintage_net_wifi, \"~> 0.7.0\"}` to your `mix.exs` dependencies.\n2. Replace all references to `VintageNet.Technology.WiFi` in your code to\n   `VintageNetWiFi`. Be aware of aliases and configuration. Also, the \"F\" is\n   capital.\n3. The `:wifi` key in the network configuration is now `:vintage_net_wifi`.\n\n## [v0.6.6] - 2019-12-01\n\n* Fixed\n  * Fix warning from Dialyzer when making wild card subscriptions. Code was also\n    added to more thoroughly validate properties paths to raise on subtle issues\n    that won't do what the programmer intends.\n\n* Added\n  * Added `VintageNet.match/1` to support \"gets\" on properties using wildcards.\n\n## [v0.6.5] - 2019-11-22\n\n* Added\n  * Support wild card subscriptions to properties. This makes it possible to\n    subscribe to things like `[\"interface\", :_, \"addresses\"]` where the `:_`\n    indicates that any value in the second position should match. That\n    particular subscription would send a message whenever an IP address anywhere\n    gets added, changed, or removed.\n\n## [v0.6.4] - 2019-10-31\n\n* Added\n  * Added the `[\"interface\", ifname, \"eap_status\"]` property for EAP\n    events. EAP is currently only supported on WiFi, but is anticipated for\n    wired Ethernet too.\n\n## [v0.6.3] - 2019-10-28\n\nThis release renames the WiFi mode names. The old names still work so it's a\nbackwards compatible update. The new names are `:ap` and `:infrastructure`\ninstead of `:host` and `:client`. These names match the mode names in the IEEE\nspecifications and usage elsewhere.\n\n* Added\n  * Support static IPv4 configurations for a default gateway and list of name\n    resolvers. See `:gateway` and `:name_servers` parameters.\n  * Support ad-hoc WiFi networking (IBSS mode)\n\n## [v0.6.2] - 2019-10-11\n\n* Added\n  * Support running a simple DNS server on an interface. This was added for WiFi\n    AP mode configuration and could be useful for other scenarios.\n  * Support DHCP server response options\n  * Support disabling configuration persistence on a per-call basis. This is for\n    temporary configurations where a reboot should not preserve the setting. For\n    example, `VintageNet.configure(\"wlan0\", config, persist: false)`\n\n## [v0.6.1] - 2019-10-02\n\n* Added\n  * Add a `current_ap` property for WiFi interfaces so that programs can get\n    information about the currently associated access point\n  * Support running a DHCP server on wired Ethernet interfaces\n  * Expose `VintageNet.WiFi.WPA2.validate_passphrase/1` so that applications can\n    reuse the WiFI passphrase validation logic. This logic follows IEEE Std\n    802.11i-2004 and validates things like proper length and character set\n\n## [v0.6.0] - 2019-09-25\n\nIMPORTANT: This release contains a LOT of changes. VintageNet is still pre-1.0\nand we're actively making API changes as we gain real world experience with it.\nPlease upgrade carefully.\n\n* Changed\n  * All IP addresses are represented as tuples. You can still specify IP\n    addresses as strings, like \"192.168.1.1\", but it will be converted to tuple\n    form. When you `get` the configuration, you'll see IP addresses as tuples.\n    This means that if you save your configuration and revert to a previous\n    version of VintageNet, the settings won't work.\n  * WiFi network configuration is always under the `:networks` key. This was\n    inconsistent. Configuration normalization will update old saved\n    configurations.\n  * Support for the IPv4 broadcast IP address has been removed. Existing support\n    was incomplete and slightly confusing, so we decided to remove it for now.\n  * All IP address subnets are represented by their prefix length. For example,\n    255.255.255.0 is recorded as a subnet with prefix length 24. Configuration\n    normalization converts subnet masks to prefix length now.\n\n* Added\n  * USB gadget support - See `VintageNet.Technology.Gadget`. It is highly likely\n    that we'll refactor USB gadget support to its own project in the future.\n  * Add `:verbose` key to configs for enabling debug messages from third party\n    applications. Currently `:verbose` controls debug output from\n    `wpa_supplicant`.\n  * Allow users to pass additional options to `MuonTrap` so that it's possible\n    to run network daemons in cgroups (among other things)\n\n* Fixed\n  * Networking daemons should all be supervised now. For example, `udhcpc`\n    previously was started by `ifup` and under many conditions, it was possible\n    to get numerous instances started simultaneously. Plus failures weren't\n    detected.\n  * No more `killall` calls to cleanup state. This had prevented network\n    technologies from being used on multiple interfaces.\n  * No more `ifupdown`. This was very convenient for getting started, but has\n    numerous flaws. Search the Internet for rants. This was replaced with direct\n    calls to `ip link` and `ip addr` and adding network daemons to supervision\n    trees.\n\n* Known issues\n  * Static IP addressing is still not implemented. It's only implemented enough\n    for WiFi AP mode and USB gadget mode to work. We hope to fix this soon.\n  * It's not possible to temporarily configure network settings. At the moment,\n    if persistence is enabled (the default), configuration updates are always\n    saved.\n\n## [v0.5.1] - 2019-09-03\n\n* Fixed\n  * Add missing PSK conversion when configuring multiple WiFi networks. This\n    fixes a bug where backup networks wouldn't connect.\n\n* Added\n  * Don't poll WiFi networks that are configured for AP mode for Internet. They\n    will never have it.\n  * Reduce the number of calls to update routing tables. Previously they were\n    unnecessarily updated on DHCP failures due to timeouts. This also removes\n    quite a bit of noise from the log.\n  * Filter out interfaces with \"Null\" technologies on them from the configured\n    list. They really aren't configured so it was confusing to see them.\n\n## [v0.5.0] - 2019-08-08\n\nBackwards incompatible change: The WiFi access point property (e.g.,\n[\"interfaces\", \"wlan0\", \"access_points\"]) is now a simple list of access point\nstructs. It was formerly a map and code using this property will need to be\nupdated.\n\n## [v0.4.1] - 2019-07-29\n\n* Added\n  * Support run-time configuration of regulatory domain\n  * Error message improvement if build system is missing pkg-config\n\n## [v0.4.0] - 2019-07-22\n\nBuild note: The fix to support AP scanning when in AP-mode (see below) required\npulling in libnl-3. All official Nerves systems have it installed since it is\nrequired by the wpa_supplicant. If you're doing host builds on Linux, you'll\nneed to run `apt install libnl-genl-3-dev`.\n\n* Added\n  * Report IP addresses in the interface properties. It's now possible to listen\n    for IP address changes on interfaces. IPv4 and IPv6 addresses are reported.\n  * Support scanning for WiFi networks when an WiFi module is in AP mode. This\n    lets you make WiFi configuration wizards. See the vintage_net_wizard\n    project.\n  * Add interface MAC addresses to the interface properties\n\n* Fixed\n  * Some WiFi adapters didn't work in AP mode since their drivers didn't support\n    the P2P interface. Raspberry Pis all support the P2P interface, but some USB\n    WiFi dongles do not. The wpa_supplicant interface code was updated to use\n    fallback to the non-P2P interface in AP mode if it wasn't available.\n\n## [v0.3.1] - 2019-06-28\n\n* Added\n  * Add null persistence implementation for devices migrating from Nerves\n    Network that already have a persistence strategy in place\n\n## [v0.3.0] - 2019-06-27\n\n* Added\n  * Support the `busybox` hex.pm package to bring in networking support if not\n    present in the Nerves system image. This enables use with the minimal\n    official Nerves images.\n  * Add Unix domain socket interface to the `wpa_supplicant`. This enables\n    much faster scanning of WiFi networks and other things like collecting\n    attached clients when in AP-mode and pinging the supplicant to make sure\n    it's running.\n  * Log output of commandline-run applications so that error messages don't get\n    lost.\n  * Provide utilities for reporting WiFi signal strength as a percent to end\n    users.\n\n* Fixed\n  * Support scanning WiFi access points with Unicode names (emoji, etc. in their\n    SSIDs)\n  * Allow internet connectivity pings to be missed 3 times in a row before\n    deciding that the internet isn't reachable. This avoids transients due to\n    the random dropped packet.\n  * Reduce externally visible transients due to internal GenServers crashing and\n    restarting - also addressed the crashes\n  * Support configure while configuring - let's you cancel a configuration that\n    takes a long time to apply and apply a new one\n\n## [v0.2.4] - 2019-06-03\n\n* Added\n  * Listen for interface additions and physical layer notifications so that\n    routing and status updates can be made much more quickly\n  * Add `lower_up` to the interface properties\n\n## [v0.2.3] - 2019-05-29\n\n* Fixed\n  * This release fixes supervision issues so that internal VintageNet crashes\n    can be recovered\n  * `VintageNet.get_configuration/1` works now\n  * `\"available_interfaces\"` is updated again\n\n## [v0.2.2] - 2019-05-24\n\n* Fixed\n  * Fix local LAN routing\n\n## [v0.2.1] - 2019-05-16\n\n* Added\n  * Expose summary status of whether the whole device is\n    disconnected, LAN-connected, or Internet-connected\n\n## [v0.2.0] - 2019-05-15\n\n* Added\n  * Support WiFi AP mode - see README.md for example\n\n* Fixed\n  * Alway update local routes before default routes to avoid getting errors when\n    Linux detects a table entry that cannot be routed","ref":"changelog.html"},{"type":"extras","title":"v0.1.0 - Changelog","doc":"Initial release to hex.\n\n[v0.13.4]: https://github.com/nerves-networking/vintage_net/compare/v0.13.3...v0.13.4\n[v0.13.3]: https://github.com/nerves-networking/vintage_net/compare/v0.13.2...v0.13.3\n[v0.13.2]: https://github.com/nerves-networking/vintage_net/compare/v0.13.1...v0.13.2\n[v0.13.1]: https://github.com/nerves-networking/vintage_net/compare/v0.13.0...v0.13.1\n[v0.13.0]: https://github.com/nerves-networking/vintage_net/compare/v0.12.2...v0.13.0\n[v0.12.2]: https://github.com/nerves-networking/vintage_net/compare/v0.12.1...v0.12.2\n[v0.12.1]: https://github.com/nerves-networking/vintage_net/compare/v0.12.0...v0.12.1\n[v0.12.0]: https://github.com/nerves-networking/vintage_net/compare/v0.11.5...v0.12.0\n[v0.11.5]: https://github.com/nerves-networking/vintage_net/compare/v0.11.4...v0.11.5\n[v0.11.4]: https://github.com/nerves-networking/vintage_net/compare/v0.11.3...v0.11.4\n[v0.11.3]: https://github.com/nerves-networking/vintage_net/compare/v0.11.2...v0.11.3\n[v0.11.2]: https://github.com/nerves-networking/vintage_net/compare/v0.11.1...v0.11.2\n[v0.11.1]: https://github.com/nerves-networking/vintage_net/compare/v0.11.0...v0.11.1\n[v0.11.0]: https://github.com/nerves-networking/vintage_net/compare/v0.10.5...v0.11.0\n[v0.10.5]: https://github.com/nerves-networking/vintage_net/compare/v0.10.4...v0.10.5\n[v0.10.4]: https://github.com/nerves-networking/vintage_net/compare/v0.10.3...v0.10.4\n[v0.10.3]: https://github.com/nerves-networking/vintage_net/compare/v0.10.2...v0.10.3\n[v0.10.2]: https://github.com/nerves-networking/vintage_net/compare/v0.10.1...v0.10.2\n[v0.10.1]: https://github.com/nerves-networking/vintage_net/compare/v0.10.0...v0.10.1\n[v0.10.0]: https://github.com/nerves-networking/vintage_net/compare/v0.9.3...v0.10.0\n[v0.9.3]: https://github.com/nerves-networking/vintage_net/compare/v0.9.2...v0.9.3\n[v0.9.2]: https://github.com/nerves-networking/vintage_net/compare/v0.9.1...v0.9.2\n[v0.9.1]: https://github.com/nerves-networking/vintage_net/compare/v0.9.0...v0.9.1\n[v0.9.0]: https://github.com/nerves-networking/vintage_net/compare/v0.8.0...v0.9.0\n[v0.8.0]: https://github.com/nerves-networking/vintage_net/compare/v0.7.9...v0.8.0\n[v0.7.9]: https://github.com/nerves-networking/vintage_net/compare/v0.7.8...v0.7.9\n[v0.7.8]: https://github.com/nerves-networking/vintage_net/compare/v0.7.7...v0.7.8\n[v0.7.7]: https://github.com/nerves-networking/vintage_net/compare/v0.7.6...v0.7.7\n[v0.7.6]: https://github.com/nerves-networking/vintage_net/compare/v0.7.5...v0.7.6\n[v0.7.5]: https://github.com/nerves-networking/vintage_net/compare/v0.7.4...v0.7.5\n[v0.7.4]: https://github.com/nerves-networking/vintage_net/compare/v0.7.3...v0.7.4\n[v0.7.3]: https://github.com/nerves-networking/vintage_net/compare/v0.7.2...v0.7.3\n[v0.7.2]: https://github.com/nerves-networking/vintage_net/compare/v0.7.1...v0.7.2\n[v0.7.1]: https://github.com/nerves-networking/vintage_net/compare/v0.7.0...v0.7.1\n[v0.7.0]: https://github.com/nerves-networking/vintage_net/compare/v0.6.6...v0.7.0\n[v0.6.6]: https://github.com/nerves-networking/vintage_net/compare/v0.6.5...v0.6.6\n[v0.6.5]: https://github.com/nerves-networking/vintage_net/compare/v0.6.4...v0.6.5\n[v0.6.4]: https://github.com/nerves-networking/vintage_net/compare/v0.6.3...v0.6.4\n[v0.6.3]: https://github.com/nerves-networking/vintage_net/compare/v0.6.2...v0.6.3\n[v0.6.2]: https://github.com/nerves-networking/vintage_net/compare/v0.6.1...v0.6.2\n[v0.6.1]: https://github.com/nerves-networking/vintage_net/compare/v0.6.0...v0.6.1\n[v0.6.0]: https://github.com/nerves-networking/vintage_net/compare/v0.5.1...v0.6.0\n[v0.5.1]: https://github.com/nerves-networking/vintage_net/compare/v0.5.0...v0.5.1\n[v0.5.0]: https://github.com/nerves-networking/vintage_net/compare/v0.4.1...v0.5.0\n[v0.4.1]: https://github.com/nerves-networking/vintage_net/compare/v0.4.0...v0.4.3\n[v0.4.0]: https://github.com/nerves-networking/vintage_net/compare/v0.3.1...v0.4.2\n[v0.3.1]: https://github.com/nerves-networking/vintage_net/compare/v0.3.0...v0.3.1\n[v0.3.0]: https://github.com/nerves-networking/vintage_net/compare/v0.2.4...v0.3.0\n[v0.2.4]: https://github.com/nerves-networking/vintage_net/compare/v0.2.3...v0.2.4\n[v0.2.3]: https://github.com/nerves-networking/vintage_net/compare/v0.2.2...v0.2.3\n[v0.2.2]: https://github.com/nerves-networking/vintage_net/compare/v0.2.1...v0.2.2\n[v0.2.1]: https://github.com/nerves-networking/vintage_net/compare/v0.2.0...v0.2.1\n[v0.2.0]: https://github.com/nerves-networking/vintage_net/compare/v0.1.0...v0.2.0","ref":"changelog.html#v0-1-0"}],"content_type":"text/markdown"}